@@ -1,5 +1,5 @@
 /*
- * Copyright 2021, 2022 Nico Sonack <nsonack@herrhotzenplotz.de>
+ * Copyright 2021-2025 Nico Sonack <nsonack@herrhotzenplotz.de>
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -40,18 +40,17 @@
 #include <stdio.h>
 #include <stdlib.h>
 
-#include <pdjson/pdjson.h>
-#include <sn/sn.h>
+#include <gcli/port/sv.h>
 
-typedef int (*parsefn)(gcli_ctx *, json_stream *stream, void *list,
-                       size_t *listsize);
+#include <pdjson/pdjson.h>
+
+typedef int (*parsefn)(struct gcli_ctx *, struct json_stream *stream,
+                       void *list, size_t *listsize);
 typedef void (*filterfn)(void *list, size_t *listsize, void const *userdata);
-typedef struct gcli_fetch_buffer gcli_fetch_buffer;
-typedef struct gcli_fetch_list_ctx gcli_fetch_list_ctx;
 
 struct gcli_fetch_buffer {
-	char   *data;
-	size_t  length;
+	char *data;
+	size_t length;
 };
 
 struct gcli_fetch_list_ctx {
@@ -64,27 +63,32 @@ struct gcli_fetch_list_ctx {
 	void const *userdata;
 };
 
-int gcli_fetch(gcli_ctx *ctx, char const *url, char **pagination_next,
-               gcli_fetch_buffer *out);
+void gcli_fetch_buffer_free(struct gcli_fetch_buffer *buffer);
 
-int gcli_curl(gcli_ctx *ctx, FILE *stream, char const *url,
+int gcli_fetch(struct gcli_ctx *ctx, char const *url, char **pagination_next,
+               struct gcli_fetch_buffer *out);
+
+int gcli_curl(struct gcli_ctx *ctx, FILE *stream, char const *url,
               char const *content_type);
 
-int gcli_fetch_with_method(gcli_ctx *ctx, char const *method,
+int gcli_fetch_with_method(struct gcli_ctx *ctx, char const *method,
                            char const *url, char const *data,
-                           char **pagination_next, gcli_fetch_buffer *out);
+                           char **pagination_next,
+                           struct gcli_fetch_buffer *out);
 
-int gcli_post_upload(gcli_ctx *ctx, char const *url, char const *content_type,
-                     void *buffer, size_t buffer_size, gcli_fetch_buffer *out);
+int gcli_post_upload(struct gcli_ctx *ctx, char const *url,
+                     char const *content_type, void *buffer, size_t buffer_size,
+                     struct gcli_fetch_buffer *out);
 
-int gcli_curl_gitea_upload_attachment(gcli_ctx *ctx, char const *url,
+int gcli_curl_gitea_upload_attachment(struct gcli_ctx *ctx, char const *url,
                                       char const *filename,
-                                      gcli_fetch_buffer *out);
+                                      struct gcli_fetch_buffer *out);
 
-int gcli_curl_test_success(gcli_ctx *ctx, char const *url);
+int gcli_curl_test_success(struct gcli_ctx *ctx, char const *url);
 char *gcli_urlencode(char const *);
-sn_sv gcli_urlencode_sv(sn_sv const);
-char *gcli_urldecode(gcli_ctx *ctx, char const *input);
-int gcli_fetch_list(gcli_ctx *ctx, char *url, gcli_fetch_list_ctx *fctx);
+gcli_sv gcli_urlencode_sv(gcli_sv const);
+char *gcli_urldecode(struct gcli_ctx *ctx, char const *input);
+int gcli_fetch_list(struct gcli_ctx *ctx, char *url,
+                    struct gcli_fetch_list_ctx *fctx);
 
 #endif /* CURL_H */
