@@ -1,5 +1,5 @@
 /*
- * Copyright 2022 Nico Sonack <nsonack@herrhotzenplotz.de>
+ * Copyright 2022-2025 Nico Sonack <nsonack@herrhotzenplotz.de>
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -59,11 +59,11 @@ usage(void)
 }
 
 void
-github_print_checks(github_check_list const *const list)
+github_print_checks(struct github_check_list const *const list)
 {
 	gcli_tbl table;
-	gcli_tblcoldef cols[] = {
-		{ .name = "ID",         .type = GCLI_TBLCOLTYPE_LONG,   .flags = GCLI_TBLCOL_JUSTIFYR },
+	struct gcli_tblcoldef cols[] = {
+		{ .name = "ID",         .type = GCLI_TBLCOLTYPE_ID,     .flags = GCLI_TBLCOL_JUSTIFYR },
 		{ .name = "STATUS",     .type = GCLI_TBLCOLTYPE_STRING, .flags = 0 },
 		{ .name = "CONCLUSION", .type = GCLI_TBLCOLTYPE_STRING, .flags = GCLI_TBLCOL_STATECOLOURED },
 		{ .name = "STARTED",    .type = GCLI_TBLCOLTYPE_STRING, .flags = 0 },
@@ -78,7 +78,7 @@ github_print_checks(github_check_list const *const list)
 
 	table = gcli_tbl_begin(cols, ARRAY_SIZE(cols));
 	if (!table)
-		errx(1, "error: could not init table");
+		errx(1, "gcli: error: could not init table");
 
 	for (size_t i = 0; i < list->checks_size; ++i) {
 		gcli_tbl_add_row(table, list->checks[i].id, list->checks[i].status,
@@ -90,13 +90,13 @@ github_print_checks(github_check_list const *const list)
 }
 
 int
-github_checks(char const *const owner, char const *const repo,
-              char const *const ref, int const max)
+github_checks(struct gcli_path const *const path, char const *const ref,
+              int const max)
 {
-	github_check_list list = {0};
+	struct github_check_list list = {0};
 	int rc = 0;
 
-	rc = github_get_checks(g_clictx, owner, repo, ref, max, &list);
+	rc = github_get_checks(g_clictx, path, ref, max, &list);
 	if (rc < 0)
 		return rc;
 
@@ -110,7 +110,7 @@ int
 subcommand_ci(int argc, char *argv[])
 {
 	int ch = 0;
-	char const *owner = NULL, *repo = NULL;
+	struct gcli_path repo_path = {0};
 	char const *ref = NULL;
 	int count = -1;             /* fetch all checks by default */
 
@@ -125,10 +125,10 @@ subcommand_ci(int argc, char *argv[])
 	while ((ch = getopt_long(argc, argv, "n:o:r:", options, NULL)) != -1) {
 		switch (ch) {
 		case 'o':
-			owner = optarg;
+			repo_path.as_default.owner = optarg;
 			break;
 		case 'r':
-			repo = optarg;
+			repo_path.as_default.repo = optarg;
 			break;
 		case 'n': {
 			char *endptr = NULL;
@@ -149,13 +149,13 @@ subcommand_ci(int argc, char *argv[])
 	/* Check that we have exactly one left argument and print proper
 	 * error messages */
 	if (argc < 1) {
-		fprintf(stderr, "error: missing ref\n");
+		fprintf(stderr, "gcli: error: missing ref\n");
 		usage();
 		return EXIT_FAILURE;
 	}
 
 	if (argc > 1) {
-		fprintf(stderr, "error: stray arguments\n");
+		fprintf(stderr, "gcli: error: stray arguments\n");
 		usage();
 		return EXIT_FAILURE;
 	}
@@ -163,16 +163,16 @@ subcommand_ci(int argc, char *argv[])
 	/* Save the ref */
 	ref = argv[0];
 
-	check_owner_and_repo(&owner, &repo);
+	check_path(&repo_path);
 
 	/* Make sure we are actually talking about a github remote because
 	 * we might be incorrectly inferring it */
 	if (gcli_config_get_forge_type(g_clictx) != GCLI_FORGE_GITHUB)
-		errx(1, "error: The ci subcommand only works for GitHub. "
+		errx(1, "gcli: error: The ci subcommand only works for GitHub. "
 		     "Use gcli -t github ... to force a GitHub remote.");
 
-	if (github_checks(owner, repo, ref, count) < 0)
-		errx(1, "error: failed to get github checks: %s",
+	if (github_checks(&repo_path, ref, count) < 0)
+		errx(1, "gcli: error: failed to get github checks: %s",
 		     gcli_get_error(g_clictx));
 
 
