@@ -1,5 +1,5 @@
 /*
- * Copyright 2021, 2022 Nico Sonack <nsonack@herrhotzenplotz.de>
+ * Copyright 2021-2025 Nico Sonack <nsonack@herrhotzenplotz.de>
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -29,13 +29,18 @@
 
 #include <config.h>
 
+#include <gcli/cmd/cmd.h>
 #include <gcli/cmd/cmdconfig.h>
 #include <gcli/cmd/gitconfig.h>
 
+#include <gcli/ctx.h>
+#include <gcli/gcli.h>
 #include <gcli/forges.h>
 #include <gcli/gitea/config.h>
 #include <gcli/github/config.h>
 #include <gcli/gitlab/config.h>
+#include <gcli/port/string.h>
+#include <gcli/port/util.h>
 
 #include <assert.h>
 #include <ctype.h>
@@ -46,26 +51,14 @@
 #include <stdlib.h>
 #include <unistd.h>
 
-#ifdef HAVE_SYS_QUEUE_H
-#include <sys/queue.h>
-#endif /* HAVE_SYS_QUEUE_H */
-
-struct gcli_config_entry {
-	TAILQ_ENTRY(gcli_config_entry) next;
-	sn_sv key;
-	sn_sv value;
-};
-
-TAILQ_HEAD(gcli_config_entries, gcli_config_entry);
 struct gcli_config_section {
 	TAILQ_ENTRY(gcli_config_section) next;
 
-    struct gcli_config_entries entries;
+	struct gcli_config_entries entries;
 
-	sn_sv title;
+	gcli_sv title;
 };
 
-#define CONFIG_MAX_SECTIONS 16
 struct gcli_config {
 	TAILQ_HEAD(gcli_config_sections, gcli_config_section) sections;
 
@@ -74,19 +67,22 @@ struct gcli_config {
 	int override_forgetype;
 	int colours_disabled;       /* NO_COLOR set or output is not a TTY */
 	int force_colours;          /* -c option was given */
+	int no_spinner;             /* don't show a progress spinner */
+	int no_markdown;            /* do not render markdown (when built with lowdown) */
+	int enable_experimental;    /* enable experimental features */
 
-	sn_sv  buffer;
-	void  *mmap_pointer;
-	bool   inited;
+	gcli_sv buffer;
+	char    *file_content;
+	bool    inited;
 };
 
 struct gcli_dotgcli {
 	struct gcli_config_entries entries;
 
-	sn_sv  buffer;
-	void  *mmap_pointer;
-	bool   has_been_searched_for;
-	bool   has_been_found;
+	gcli_sv buffer;
+	char    *file_content;
+	bool    has_been_searched_for;
+	bool    has_been_found;
 };
 
 struct cmd_ctx {
@@ -109,7 +105,7 @@ ctx_config(struct gcli_ctx *ctx)
 }
 
 static bool
-should_init_dotgcli(gcli_ctx *ctx)
+should_init_dotgcli(struct gcli_ctx *ctx)
 {
 	struct gcli_dotgcli *dgcli = ctx_dotgcli(ctx);
 
@@ -127,7 +123,7 @@ find_dotgcli(void)
 
 	curr_dir_path = getcwd(NULL, 128);
 	if (!curr_dir_path)
-		err(1, "getcwd");
+		err(1, "gcli: getcwd");
 
 	/* Here we are trying to traverse upwards through the directory
 	 * tree, searching for a directory called .git.
@@ -135,7 +131,7 @@ find_dotgcli(void)
 	do {
 		curr_dir = opendir(curr_dir_path);
 		if (!curr_dir)
-			err(1, "opendir");
+			err(1, "gcli: opendir");
 
 		while ((ent = readdir(curr_dir))) {
 			if (strcmp(".", ent->d_name) == 0 || strcmp("..", ent->d_name) == 0)
@@ -164,9 +160,9 @@ find_dotgcli(void)
 
 			free(curr_dir_path);
 
-			curr_dir_path = realpath(tmp, NULL);
+			curr_dir_path = gcli_cmd_realpath(tmp);
 			if (!curr_dir_path)
-				err(1, "realpath at %s", tmp);
+				err(1, "gcli: realpath at %s", tmp);
 
 			free(tmp);
 
@@ -190,7 +186,7 @@ find_dotgcli(void)
 }
 
 static void
-init_local_config(gcli_ctx *ctx)
+init_local_config(struct gcli_ctx *ctx)
 {
 	if (!should_init_dotgcli(ctx)) {
 		return;
@@ -208,49 +204,49 @@ init_local_config(gcli_ctx *ctx)
 	dgcli->has_been_searched_for = true;
 	dgcli->has_been_found = true;
 
-	int len = sn_mmap_file(path, &dgcli->mmap_pointer);
+	int len = gcli_read_file(path, &dgcli->file_content);
 	if (len < 0)
-		err(1, "Unable to open config file");
+		err(1, "gcli: unable to open config file");
 
-	dgcli->buffer = sn_sv_from_parts(dgcli->mmap_pointer, len);
-	dgcli->buffer = sn_sv_trim_front(dgcli->buffer);
+	dgcli->buffer = gcli_sv_from_parts(dgcli->file_content, len);
+	dgcli->buffer = gcli_sv_trim_front(dgcli->buffer);
 
 	int curr_line = 1;
 	while (dgcli->buffer.length > 0) {
-		sn_sv line = sn_sv_chop_until(&dgcli->buffer, '\n');
+		gcli_sv line = gcli_sv_chop_until(&dgcli->buffer, '\n');
 
-		line = sn_sv_trim(line);
+		line = gcli_sv_trim(line);
 
 		if (line.length == 0)
-			errx(1, "%s:%d: Unexpected end of line",
+			errx(1, "gcli: %s:%d: Unexpected end of line",
 			     path, curr_line);
 
 		// Comments
 		if (line.data[0] == '#') {
-			dgcli->buffer = sn_sv_trim_front(dgcli->buffer);
+			dgcli->buffer = gcli_sv_trim_front(dgcli->buffer);
 			curr_line++;
 			continue;
 		}
 
-		sn_sv key = sn_sv_chop_until(&line, '=');
+		gcli_sv key = gcli_sv_chop_until(&line, '=');
 
-		key = sn_sv_trim(key);
+		key = gcli_sv_trim(key);
 
 		if (key.length == 0)
-			errx(1, "%s:%d: empty key", path, curr_line);
+			errx(1, "gcli: %s:%d: empty key", path, curr_line);
 
 		line.data   += 1;
 		line.length -= 1;
 
-		sn_sv value = sn_sv_trim(line);
+		gcli_sv value = gcli_sv_trim(line);
 
-		struct gcli_config_entry *entry = calloc(sizeof(*entry), 1);
+		struct gcli_config_entry *entry = calloc(1, sizeof(*entry));
 		TAILQ_INSERT_TAIL(&dgcli->entries, entry, next);
 
 	    entry->key = key;
 	    entry->value = value;
 
-		dgcli->buffer = sn_sv_trim_front(dgcli->buffer);
+		dgcli->buffer = gcli_sv_trim_front(dgcli->buffer);
 		curr_line++;
 	}
 
@@ -258,7 +254,7 @@ init_local_config(gcli_ctx *ctx)
 }
 
 struct config_parser {
-	sn_sv buffer;
+	gcli_sv buffer;
 	int line;
 	char const *filename;
 };
@@ -288,7 +284,7 @@ again:
 not_whitespace:
 	if (input->buffer.data[0] == '#') {
 		/* This is a comment */
-		sn_sv_chop_until(&input->buffer, '\n');
+		gcli_sv_chop_until(&input->buffer, '\n');
 		goto again;
 	}
 }
@@ -297,46 +293,46 @@ static void
 parse_section_entry(struct config_parser *input,
                     struct gcli_config_section *section)
 {
-	struct gcli_config_entry *entry = calloc(sizeof(*entry), 1);
+	struct gcli_config_entry *entry = calloc(1, sizeof(*entry));
 	TAILQ_INSERT_TAIL(&section->entries, entry, next);
 
-	sn_sv key = sn_sv_chop_until(&input->buffer, '=');
+	gcli_sv key = gcli_sv_chop_until(&input->buffer, '=');
 
 	if (key.length == 0)
-		errx(1, "%s:%d: empty key", input->filename, input->line);
+		errx(1, "gcli: %s:%d: empty key", input->filename, input->line);
 
 	input->buffer.data   += 1;
 	input->buffer.length -= 1;
 
-	sn_sv value = sn_sv_chop_until(&input->buffer, '\n');
+	gcli_sv value = gcli_sv_chop_until(&input->buffer, '\n');
 
-	entry->key   = sn_sv_trim(key);
-	entry->value = sn_sv_trim(value);
+	entry->key   = gcli_sv_trim(key);
+	entry->value = gcli_sv_trim(value);
 }
 
-static sn_sv
+static gcli_sv
 parse_section_title(struct config_parser *input)
 {
 	size_t len = 0;
 	if (input->buffer.length == 0)
-		errx(1, "%s:%d: unexpected end of input in section title",
+		errx(1, "gcli: %s:%d: unexpected end of input in section title",
 		     input->filename, input->line);
 
 
 	while (!isspace(input->buffer.data[len]) && input->buffer.data[len] != '{')
 		len++;
 
-	sn_sv title = sn_sv_from_parts(input->buffer.data, len);
+	gcli_sv title = gcli_sv_from_parts(input->buffer.data, len);
 	input->buffer.data   += len;
 	input->buffer.length -= len;
 
 	skip_ws_and_comments(input);
 
 	if (input->buffer.length == 0)
-		errx(1, "%s:%d: unexpected end of input", input->filename, input->line);
+		errx(1, "gcli: %s:%d: unexpected end of input", input->filename, input->line);
 
 	if (input->buffer.data[0] != '{')
-		errx(1, "%s:%d: expected '{'", input->filename, input->line);
+		errx(1, "gcli: %s:%d: expected '{'", input->filename, input->line);
 
 	input->buffer.length -= 1;
 	input->buffer.data   += 1;
@@ -351,7 +347,7 @@ parse_config_section(struct gcli_config *cfg,
 {
 	struct gcli_config_section *section = NULL;
 
-	section = calloc(sizeof(*section), 1);
+	section = calloc(1, sizeof(*section));
 	TAILQ_INSERT_TAIL(&cfg->sections, section, next);
 
 	section->title = parse_section_title(input);
@@ -366,7 +362,7 @@ parse_config_section(struct gcli_config *cfg,
 	}
 
 	if (input->buffer.length == 0)
-		errx(1, "%s:%d: missing '}' before end of file",
+		errx(1, "gcli: %s:%d: missing '}' before end of file",
 		     input->filename, input->line);
 
 	input->buffer.length -= 1;
@@ -390,7 +386,7 @@ parse_config_file(struct gcli_config *cfg,
  * return 0. Otherwise return -1.
  */
 static struct gcli_config *
-ensure_config(gcli_ctx *ctx)
+ensure_config(struct gcli_ctx *ctx)
 {
 	struct gcli_config *cfg = ctx_config(ctx);
 	char *file_path = NULL;
@@ -401,32 +397,32 @@ ensure_config(gcli_ctx *ctx)
 
 	cfg->inited = true;
 
-	file_path = getenv("XDG_CONFIG_PATH");
+	file_path = getenv("XDG_CONFIG_HOME");
 	if (!file_path) {
 		file_path = getenv("HOME");
 		if (!file_path) {
-			warnx("Neither XDG_CONFIG_PATH nor HOME set in env");
+			gcli_warnx(ctx, "Neither XDG_CONFIG_HOME nor HOME set in env");
 			return cfg;
 		}
 
 		/*
 		 * Code duplication to avoid leaking pointers */
-		file_path = sn_asprintf("%s/.config/gcli/config", file_path);
+		file_path = gcli_asprintf("%s/.config/gcli/config", file_path);
 	} else {
-		file_path = sn_asprintf("%s/gcli/config", file_path);
+		file_path = gcli_asprintf("%s/gcli/config", file_path);
 	}
 
 	if (access(file_path, R_OK) < 0) {
-		warn("Cannot access config file at %s", file_path);
+		gcli_warn(ctx, "gcli: cannot access config file at %s", file_path);
 		return cfg;
 	}
 
-	int len = sn_mmap_file(file_path, &cfg->mmap_pointer);
+	int len = gcli_read_file(file_path, &cfg->file_content);
 	if (len < 0)
-		err(1, "Unable to open config file");
+		err(1, "gcli: unable to open config file");
 
-	cfg->buffer = sn_sv_from_parts(cfg->mmap_pointer, len);
-	cfg->buffer = sn_sv_trim_front(cfg->buffer);
+	cfg->buffer = gcli_sv_from_parts(cfg->file_content, len);
+	cfg->buffer = gcli_sv_trim_front(cfg->buffer);
 
 	parser.buffer   = cfg->buffer;
 	parser.line     = 1;
@@ -441,22 +437,29 @@ ensure_config(gcli_ctx *ctx)
 
 /** Check input for a value that indicates yes/true */
 static int
-checkyes(char const *const tmp)
+string_means_true(char const *const tmp)
 {
-	static char const *const yeses[] = { "1", "y", "Y" };
+	size_t tmplen = strlen(tmp) + 1;
+	char *tmp_lower = malloc(tmplen);
 
-	if (strlen(tmp) == 3) {
-		if (tolower(tmp[0]) == 'y' && tolower(tmp[1]) == 'e' &&
-		    tolower(tmp[2]) == 's')
-			return 1;
+	strncpy(tmp_lower, tmp, tmplen);
+
+	for (size_t i = 0; i < tmplen - 1; ++i) {
+		tmp_lower[i] = tolower(tmp_lower[i]);
 	}
 
-	for (size_t i = 0; i < ARRAY_SIZE(yeses); ++i) {
-		if (strcmp(yeses[i], tmp) == 0)
-			return 1;
-	}
+	int is_yes = strcmp(tmp_lower, "1") == 0 ||
+		strcmp(tmp_lower, "yes") == 0 ||
+		strcmp(tmp_lower, "true") == 0;
 
-	return 0;
+	free(tmp_lower);
+	return is_yes;
+}
+
+static bool
+string_means_false(char const *const tmp)
+{
+	return !string_means_true(tmp);
 }
 
 /* readenv: Read values of environment variables and pre-populate the
@@ -481,13 +484,22 @@ readenv(struct gcli_config *cfg)
 	 * violate the definition to get expected and sane behaviour. */
 	tmp = getenv("NO_COLOR");
 	if (tmp && tmp[0] != '\0')
-		cfg->colours_disabled = checkyes(tmp);
+		cfg->colours_disabled = string_means_true(tmp);
+
+	if ((tmp = getenv("GCLI_NOSPINNER")))
+		cfg->no_spinner = string_means_true(tmp);
+
+	if ((tmp = getenv("GCLI_RENDER_MARKDOWN")))
+		cfg->no_markdown = string_means_false(tmp);
+
+	if ((tmp = getenv("GCLI_ENABLE_EXPERIMENTAL")))
+		cfg->enable_experimental = string_means_true(tmp);
 }
 
 int
 gcli_config_init_ctx(struct gcli_ctx *ctx)
 {
-	struct cmd_ctx *cctx = calloc(sizeof(*cctx), 1);
+	struct cmd_ctx *cctx = calloc(1, sizeof(*cctx));
 	gcli_set_userdata(ctx, cctx);
 
 	cctx->config.sections =
@@ -502,7 +514,7 @@ gcli_config_init_ctx(struct gcli_ctx *ctx)
 }
 
 int
-gcli_config_parse_args(gcli_ctx *ctx, int *argc, char ***argv)
+gcli_config_parse_args(struct gcli_ctx *ctx, int *argc, char ***argv)
 {
 	/* These are the very first options passed to the gcli command
 	 * itself. It is the first ever getopt call we do to parse any
@@ -525,6 +537,14 @@ gcli_config_parse_args(gcli_ctx *ctx, int *argc, char ***argv)
 		  .has_arg = no_argument,
 		  .flag    = &cfg->colours_disabled,
 		  .val     = 0 },
+		{ .name    = "no-spinner",
+		  .has_arg = no_argument,
+		  .flag    = &cfg->no_spinner,
+		  .val     = 1 },
+		{ .name    = "no-markdown",
+		  .has_arg = no_argument,
+		  .flag    = &cfg->no_markdown,
+		  .val     = 1 },
 		{ .name    = "type",
 		  .has_arg = required_argument,
 		  .flag    = NULL,
@@ -537,11 +557,15 @@ gcli_config_parse_args(gcli_ctx *ctx, int *argc, char ***argv)
 		  .has_arg = no_argument,
 		  .flag    = NULL,
 		  .val     = 'v' },
+		{ .name    = "version",
+		  .has_arg = no_argument,
+		  .flag    = NULL,
+		  .val     = 'V' },
 		{0},
 	};
 
 	/* by default we are not verbose */
-	sn_setverbosity(VERBOSITY_NORMAL);
+	gcli_setverbosity(g_clictx, GCLI_VERBOSITY_NORMAL);
 
 	/* Before we parse options, invalidate the override type so it
 	 * doesn't get confused later */
@@ -550,7 +574,7 @@ gcli_config_parse_args(gcli_ctx *ctx, int *argc, char ***argv)
 	/* Start off by pre-populating the config structure */
 	readenv(cfg);
 
-	while ((ch = getopt_long(*argc, *argv, "+a:r:cqvt:", options, NULL)) != -1) {
+	while ((ch = getopt_long(*argc, *argv, "+a:r:cqvt:V", options, NULL)) != -1) {
 		switch (ch) {
 		case 'a': {
 			cfg->override_default_account = optarg;
@@ -562,10 +586,10 @@ gcli_config_parse_args(gcli_ctx *ctx, int *argc, char ***argv)
 			cfg->force_colours = 1;
 		} break;
 		case 'q': {
-			sn_setverbosity(VERBOSITY_QUIET);
+			gcli_setverbosity(ctx, GCLI_VERBOSITY_QUIET);
 		} break;
 		case 'v': {
-			sn_setverbosity(VERBOSITY_VERBOSE);
+			gcli_setverbosity(ctx, GCLI_VERBOSITY_VERBOSE);
 		} break;
 		case 't': {
 			if (strcmp(optarg, "github") == 0) {
@@ -574,12 +598,21 @@ gcli_config_parse_args(gcli_ctx *ctx, int *argc, char ***argv)
 				cfg->override_forgetype = GCLI_FORGE_GITLAB;
 			} else if (strcmp(optarg, "gitea") == 0) {
 				cfg->override_forgetype = GCLI_FORGE_GITEA;
+			} else if (strcmp(optarg, "bugzilla") == 0) {
+				cfg->override_forgetype = GCLI_FORGE_BUGZILLA;
 			} else {
-				fprintf(stderr, "error: unknown forge type '%s'. "
+				fprintf(stderr, "gcli: error: unknown forge type '%s'. "
 				        "Have either github, gitlab or gitea.\n", optarg);
 				return EXIT_FAILURE;
 			}
 		} break;
+		case 'V': {
+			longversion();
+			/* call exit here because if we return an OK we would continue
+			 * running the gcli command. we do not want this as this flag
+			 * only ever prints the version and exits. */
+			exit(EXIT_SUCCESS);
+		} break;
 		case 0: break;
 		case '?':
 		default:
@@ -609,14 +642,30 @@ find_section(struct gcli_config *cfg, char const *name)
 	struct gcli_config_section *section;
 
 	TAILQ_FOREACH(section, &cfg->sections, next) {
-		if (sn_sv_eq_to(section->title, name))
+		if (gcli_sv_eq_to(section->title, name))
 			return section;
 	}
 	return NULL;
 }
 
-sn_sv
-gcli_config_find_by_key(gcli_ctx *ctx, char const *section_name, char const *key)
+struct gcli_config_entries const *
+gcli_config_get_section_entries(struct gcli_ctx *ctx, char const *section_name)
+{
+	struct gcli_config_section const *s;
+	struct gcli_config *cfg;
+
+	cfg = ensure_config(ctx);
+	s = find_section(cfg, section_name);
+
+	if (s == NULL)
+		return NULL;
+	else
+		return &s->entries;
+}
+
+gcli_sv
+gcli_config_find_by_key(struct gcli_ctx *ctx, char const *section_name,
+                        char const *key)
 {
 	struct gcli_config_entry *entry;
 	struct gcli_config *cfg = ensure_config(ctx);
@@ -625,26 +674,26 @@ gcli_config_find_by_key(gcli_ctx *ctx, char const *section_name, char const *key
 		find_section(cfg, section_name);
 
 	if (!section) {
-		warnx("no config section with name '%s'", section_name);
+		gcli_warnx(ctx, "gcli: no config section with name '%s'", section_name);
 		return SV_NULL;
 	}
 
 	TAILQ_FOREACH(entry, &section->entries, next) {
-		if (sn_sv_eq_to(entry->key, key))
+		if (gcli_sv_eq_to(entry->key, key))
 			return entry->value;
 	}
 
 	return SV_NULL;
 }
 
-static sn_sv
-gcli_local_config_find_by_key(gcli_ctx *ctx, char const *const key)
+static gcli_sv
+gcli_local_config_find_by_key(struct gcli_ctx *ctx, char const *const key)
 {
 	struct gcli_dotgcli *lcfg = ctx_dotgcli(ctx);
 	struct gcli_config_entry *entry;
 
 	TAILQ_FOREACH(entry, &lcfg->entries, next) {
-		if (sn_sv_eq_to(entry->key, key))
+		if (gcli_sv_eq_to(entry->key, key))
 			return entry->value;
 	}
 
@@ -652,40 +701,59 @@ gcli_local_config_find_by_key(gcli_ctx *ctx, char const *const key)
 }
 
 char *
-gcli_config_get_editor(gcli_ctx *ctx)
+gcli_config_get_editor(struct gcli_ctx *ctx)
 {
 	ensure_config(ctx);
 
-	return sn_sv_to_cstr(gcli_config_find_by_key(ctx, "defaults", "editor"));
+	return gcli_sv_to_cstr(gcli_config_find_by_key(ctx, "defaults", "editor"));
+}
+
+char *
+gcli_config_get_pager(struct gcli_ctx *ctx)
+{
+	ensure_config(ctx);
+
+	return gcli_sv_to_cstr(gcli_config_find_by_key(ctx, "defaults", "pager"));
+}
+
+char *
+gcli_config_get_url_open_program(struct gcli_ctx *ctx)
+{
+	ensure_config(ctx);
+
+	return gcli_sv_to_cstr(
+		gcli_config_find_by_key(ctx, "defaults", "url-open-program")
+	);
 }
 
 static char const *const
 default_account_entry_names[] = {
-	[GCLI_FORGE_GITHUB] = "github-default-account",
-	[GCLI_FORGE_GITLAB] = "gitlab-default-account",
-	[GCLI_FORGE_GITEA]  = "gitea-default-account", };
+	[GCLI_FORGE_GITHUB]   = "github-default-account",
+	[GCLI_FORGE_GITLAB]   = "gitlab-default-account",
+	[GCLI_FORGE_GITEA]    = "gitea-default-account",
+	[GCLI_FORGE_BUGZILLA] = "bugzilla-default-account",};
 
-static char const *
-get_default_account(gcli_ctx *ctx, gcli_forge_type ftype)
+static char *
+get_default_account(struct gcli_ctx *ctx, gcli_forge_type ftype)
 {
 	char const *const defaultname = default_account_entry_names[ftype];
-	sn_sv act = gcli_config_find_by_key(ctx, "defaults", defaultname);
+	gcli_sv act = gcli_config_find_by_key(ctx, "defaults", defaultname);
 
 	if (!act.length)
 		return NULL;
 
-	return sn_sv_to_cstr(act);
+	return gcli_sv_to_cstr(act);
 }
 
-static char const *
-gcli_config_get_account(gcli_ctx *ctx)
+static char *
+gcli_config_get_account(struct gcli_ctx *ctx)
 {
 	struct gcli_config *cfg = ctx_config(ctx);
 	gcli_forge_type ftype = gcli_config_get_forge_type(ctx);
-	char const *account;
+	char *account;
 
 	if (cfg->override_default_account) {
-		account = cfg->override_default_account;
+		account = strdup(cfg->override_default_account);
 	} else {
 		account = get_default_account(ctx, ftype);
 	}
@@ -694,44 +762,64 @@ gcli_config_get_account(gcli_ctx *ctx)
 }
 
 static char const *const default_urls[] = {
-	[GCLI_FORGE_GITHUB] = "https://api.github.com",
-	[GCLI_FORGE_GITLAB] = "https://gitlab.com/api/v4",
-	[GCLI_FORGE_GITEA]  = "https://codeberg.org/api/v1",
+	[GCLI_FORGE_GITHUB]   = "https://api.github.com",
+	[GCLI_FORGE_GITLAB]   = "https://gitlab.com/api/v4",
+	[GCLI_FORGE_GITEA]    = "https://codeberg.org/api/v1",
+	[GCLI_FORGE_BUGZILLA] = "https://bugs.freebsd.org/bugzilla",
 };
 
 char *
-gcli_config_get_apibase(gcli_ctx *ctx)
+gcli_config_get_apibase(struct gcli_ctx *ctx)
 {
-	char const *acct = gcli_config_get_account(ctx);
+	char *acct = gcli_config_get_account(ctx);
 	char *url = NULL;
 
 	if (acct) {
-		sn_sv url_sv = gcli_config_find_by_key(ctx, acct, "apibase");
-		if (url_sv.length)
-			url = sn_sv_to_cstr(url_sv);
+		gcli_sv url_sv = {0};
+
+		url_sv = gcli_config_find_by_key(ctx, acct, "api-base");
+
+		/* https://github.com/herrhotzenplotz/gcli/issues/138
+		 *
+		 * Above is correct behaviour. Below is bad/buggy behaviour.
+		 * Check for the second (buggy) option apibase and
+		 * use it if needed. */
+		if (gcli_sv_null(url_sv))
+			url_sv = gcli_config_find_by_key(ctx, acct, "apibase");
+
+		if (!gcli_sv_null(url_sv))
+			url = gcli_sv_to_cstr(url_sv);
 	}
 
 	if (!url)
 		url = strdup(default_urls[gcli_config_get_forge_type(ctx)]);
 
+
+	free(acct);
+
 	return url;
 }
 
 char *
-gcli_config_get_account_name(gcli_ctx *ctx)
+gcli_config_get_account_name(struct gcli_ctx *ctx)
 {
-	char const *account = gcli_config_get_account(ctx);
-	sn_sv actname = gcli_config_find_by_key(
-		ctx, account, "account");
+	char *account = gcli_config_get_account(ctx);
+	gcli_sv actname;
 
-	return sn_sv_to_cstr(actname);
+	if (!account)
+		return NULL;
+
+	actname = gcli_config_find_by_key(ctx, account, "account");
+	free(account);
+
+	return gcli_sv_to_cstr(actname);
 }
 
 static char *
-get_account_token(gcli_ctx *ctx)
+get_account_token(struct gcli_ctx *ctx)
 {
-	char const *account;
-	sn_sv token;
+	char *account;
+	gcli_sv token;
 
 	account = gcli_config_get_account(ctx);
 	if (!account)
@@ -739,19 +827,21 @@ get_account_token(gcli_ctx *ctx)
 
 	token = gcli_config_find_by_key(ctx, account, "token");
 
-	return sn_sv_to_cstr(token);
+	free(account);
+
+	return gcli_sv_to_cstr(token);
 }
 
 char *
-gcli_config_get_token(gcli_ctx *ctx)
+gcli_config_get_token(struct gcli_ctx *ctx)
 {
 	ensure_config(ctx);
 
 	return get_account_token(ctx);
 }
 
-sn_sv
-gcli_config_get_upstream(gcli_ctx *ctx)
+gcli_sv
+gcli_config_get_upstream(struct  gcli_ctx *ctx)
 {
 	init_local_config(ctx);
 
@@ -759,45 +849,45 @@ gcli_config_get_upstream(gcli_ctx *ctx)
 }
 
 bool
-gcli_config_pr_inhibit_delete_source_branch(gcli_ctx *ctx)
+gcli_config_pr_inhibit_delete_source_branch(struct gcli_ctx *ctx)
 {
-	sn_sv val;
+	gcli_sv val;
 
 	init_local_config(ctx);
 
 	val = gcli_local_config_find_by_key(ctx, "pr.inhibit-delete-source-branch");
 
-	return sn_sv_eq_to(val,	"yes");
+	return gcli_sv_eq_to(val,	"yes");
 }
 
 void
-gcli_config_get_upstream_parts(gcli_ctx *ctx, sn_sv *const owner,
-                               sn_sv *const repo)
+gcli_config_get_upstream_parts(struct gcli_ctx *ctx, gcli_sv *const owner,
+                               gcli_sv *const repo)
 {
 	ensure_config(ctx);
 
-	sn_sv upstream = gcli_config_get_upstream(ctx);
-	*owner = sn_sv_chop_until(&upstream, '/');
+	gcli_sv upstream = gcli_config_get_upstream(ctx);
+	*owner = gcli_sv_chop_until(&upstream, '/');
 
 	/* Sanity check: did we actually reach the '/'? */
 	if (*upstream.data != '/')
-		errx(1, ".gcli has invalid upstream format. expected owner/repo");
+		errx(1, "gcli: .gcli has invalid upstream format. expected owner/repo");
 
 	upstream.data   += 1;
 	upstream.length -= 1;
 	*repo            = upstream;
 }
 
-sn_sv
-gcli_config_get_base(gcli_ctx *ctx)
+gcli_sv
+gcli_config_get_base(struct gcli_ctx *ctx)
 {
 	init_local_config(ctx);
 
 	return gcli_local_config_find_by_key(ctx, "pr.base");
 }
 
-sn_sv
-gcli_config_get_override_default_account(gcli_ctx *ctx)
+gcli_sv
+gcli_config_get_override_default_account(struct gcli_ctx *ctx)
 {
 	struct gcli_config *cfg;
 
@@ -811,7 +901,7 @@ gcli_config_get_override_default_account(gcli_ctx *ctx)
 }
 
 static gcli_forge_type
-gcli_config_get_forge_type_internal(gcli_ctx *ctx)
+gcli_config_get_forge_type_internal(struct gcli_ctx *ctx)
 {
 	struct gcli_config *cfg = ctx_config(ctx);
 
@@ -822,111 +912,137 @@ gcli_config_get_forge_type_internal(gcli_ctx *ctx)
 	ensure_config(ctx);
 	init_local_config(ctx);
 
-	sn_sv entry = {0};
+	gcli_sv entry = {0};
 
 	if (cfg->override_default_account) {
 	    char const *section = cfg->override_default_account;
 		entry = gcli_config_find_by_key(ctx, section, "forge-type");
-		if (sn_sv_null(entry))
+		if (gcli_sv_null(entry))
 			errx(1,
-			     "error: given default override account not found or "
+			     "gcli: error: given default override account not found or "
 			     "missing forge-type");
 	} else {
 		entry = gcli_local_config_find_by_key(ctx, "forge-type");
 	}
 
-	if (!sn_sv_null(entry)) {
-		if (sn_sv_eq_to(entry, "github"))
+	if (!gcli_sv_null(entry)) {
+		if (gcli_sv_eq_to(entry, "github"))
 			return GCLI_FORGE_GITHUB;
-		else if (sn_sv_eq_to(entry, "gitlab"))
+		else if (gcli_sv_eq_to(entry, "gitlab"))
 			return GCLI_FORGE_GITLAB;
-		else if (sn_sv_eq_to(entry, "gitea"))
+		else if (gcli_sv_eq_to(entry, "gitea"))
 			return GCLI_FORGE_GITEA;
+		else if (gcli_sv_eq_to(entry, "bugzilla"))
+			return GCLI_FORGE_BUGZILLA;
 		else
-			errx(1, "Unknown forge type "SV_FMT, SV_ARGS(entry));
+			errx(1, "gcli: unknown forge type "SV_FMT, SV_ARGS(entry));
 	}
 
 	/* As a last resort, try to infer from the git remote */
 	int const type = gcli_gitconfig_get_forgetype(ctx, cfg->override_remote);
 	if (type < 0)
-		errx(1, "error: cannot infer forge type. "
+		errx(1, "gcli: error: cannot infer forge type. "
 		     "use -t <forge-type> to overrride manually.");
 
 	return type;
 }
 
 gcli_forge_type
-gcli_config_get_forge_type(gcli_ctx *ctx)
+gcli_config_get_forge_type(struct gcli_ctx *ctx)
 {
 	gcli_forge_type const result = gcli_config_get_forge_type_internal(ctx);
 
 	/* print the type if verbose */
-	if (sn_verbose()) {
+	if (gcli_be_verbose(ctx)) {
 		static int have_printed_forge_type = 0;
 		static char const *const ftype_name[] = {
 			[GCLI_FORGE_GITHUB] = "GitHub",
-			[GCLI_FORGE_GITLAB] = "Gitlab",
-			[GCLI_FORGE_GITEA]  = "Gitea",
-			};
+			[GCLI_FORGE_GITLAB] = "GitLab",
+			[GCLI_FORGE_GITEA] = "Gitea",
+			[GCLI_FORGE_BUGZILLA] = "Bugzilla",
+		};
 
 		if (!have_printed_forge_type) {
 			have_printed_forge_type = 1;
-			fprintf(stderr, "info: forge type is %s\n", ftype_name[result]);
+			fprintf(stderr, "gcli: info: forge type is %s\n", ftype_name[result]);
 		}
 	}
 
 	return result;
 }
 
-void
-gcli_config_get_repo(gcli_ctx *ctx, char const **const owner,
+int
+gcli_config_get_remote(struct gcli_ctx *ctx, char **remote)
+{
+	struct gcli_config *cfg;
+	gcli_forge_type type;
+	int rc;
+
+	cfg = ensure_config(ctx);
+
+	if (cfg->override_remote) {
+		*remote = strdup(cfg->override_remote);
+		return 0;
+	}
+
+	type = gcli_config_get_forge_type(ctx);
+	rc = gcli_gitconfig_get_remote(ctx, type, remote);
+
+	return rc;
+}
+
+int
+gcli_config_get_repo(struct gcli_ctx *ctx, char const **const owner,
                      char const **const repo)
 {
-	sn_sv upstream = {0};
+	gcli_sv upstream = {0};
 	struct gcli_config *cfg;
 
 	cfg = ensure_config(ctx);
 
 	if (cfg->override_remote) {
-		int const forge = gcli_gitconfig_repo_by_remote(
-			cfg->override_remote, owner, repo);
+		int forge = 0, rc = 0;
+
+		rc = gcli_gitconfig_repo_by_remote(ctx, cfg->override_remote, owner,
+		                                   repo, &forge);
+
+		if (rc < 0)
+			return rc;
 
 		if (forge >= 0) {
 			if ((int)(gcli_config_get_forge_type(ctx)) != forge)
-				errx(1, "error: forge types are inconsistent");
+				return gcli_error(ctx, "forge types are inconsistent");
 		}
 
-		return;
+		return 0;
 	}
 
 	if ((upstream = gcli_config_get_upstream(ctx)).length != 0) {
-		sn_sv const owner_sv = sn_sv_chop_until(&upstream, '/');
-		sn_sv const repo_sv  = sn_sv_from_parts(
-			upstream.data + 1,
-			upstream.length - 1);
+		gcli_sv const owner_sv = gcli_sv_chop_until(&upstream, '/');
+		gcli_sv const repo_sv = gcli_sv_from_parts(upstream.data + 1, upstream.length - 1);
 
-		*owner = sn_sv_to_cstr(owner_sv);
-		*repo  = sn_sv_to_cstr(repo_sv);
+		*owner = gcli_sv_to_cstr(owner_sv);
+		*repo  = gcli_sv_to_cstr(repo_sv);
 
-		return;
+		return 0;
 	}
 
-	gcli_gitconfig_repo_by_remote(NULL, owner, repo);
+	return gcli_gitconfig_repo_by_remote(ctx, NULL, owner, repo, NULL);
 }
 
-int
-gcli_config_have_colours(gcli_ctx *ctx)
+bool
+gcli_config_have_colours(struct gcli_ctx *ctx)
 {
-	static int tested_tty = 0;
+	static bool tested_tty = 0;
 	struct gcli_config *cfg;
 
 	cfg = ctx_config(ctx);
 
 	if (cfg->force_colours)
-		return 1;
+		return true;
 
 	if (cfg->colours_disabled)
-		return 0;
+		return false;
 
 	if (tested_tty)
 		return !cfg->colours_disabled;
@@ -936,7 +1052,69 @@ gcli_config_have_colours(gcli_ctx *ctx)
 	else
 		cfg->colours_disabled = true;
 
-	tested_tty = 1;
+	tested_tty = true;
 
 	return !cfg->colours_disabled;
 }
+
+bool
+gcli_config_display_progress_spinner(struct gcli_ctx *ctx)
+{
+	ensure_config(ctx);
+
+	struct gcli_config *cfg;
+	cfg = ctx_config(ctx);
+
+	if (cfg->no_spinner)
+		return false;
+
+	gcli_sv cfg_entry = gcli_config_find_by_key(ctx, "defaults", "disable-spinner");
+	if (gcli_sv_null(cfg_entry))
+		return true;
+
+	if (string_means_true(gcli_sv_to_cstr(cfg_entry)))
+		return false;
+
+	return true;
+}
+
+bool
+gcli_config_render_markdown(struct gcli_ctx *ctx)
+{
+	ensure_config(ctx);
+
+	struct gcli_config *cfg;
+	cfg = ctx_config(ctx);
+
+	if (cfg->no_markdown)
+		return false;
+
+	gcli_sv cfg_entry = gcli_config_find_by_key(ctx, "defaults", "render-markdown");
+	if (gcli_sv_null(cfg_entry))
+		return true;
+
+	if (string_means_false(gcli_sv_to_cstr(cfg_entry))) {
+		cfg->no_markdown = true;
+		return false;
+	}
+
+	return true;
+}
+
+bool
+gcli_config_enable_experimental(struct gcli_ctx *ctx)
+{
+	ensure_config(ctx);
+
+	struct gcli_config *cfg;
+	cfg = ctx_config(ctx);
+
+	if (cfg->enable_experimental)
+		return true;
+
+	gcli_sv cfg_entry = gcli_config_find_by_key(ctx, "defaults", "enable-experimental");
+	if (gcli_sv_null(cfg_entry))
+		return false;
+
+	return string_means_true(gcli_sv_to_cstr(cfg_entry));
+}
