@@ -1,5 +1,5 @@
 /*
- * Copyright 2022 Nico Sonack <nsonack@herrhotzenplotz.de>
+ * Copyright 2022-2025 Nico Sonack <nsonack@herrhotzenplotz.de>
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -36,11 +36,12 @@
 #include <gcli/cmd/editor.h>
 
 #include <gcli/comments.h>
+#include <gcli/date_time.h>
 #include <gcli/json_util.h>
+#include <gcli/port/util.h>
 
-#ifdef HAVE_GETOPT_H
+#include <assert.h>
 #include <getopt.h>
-#endif
 
 static void
 usage(void)
@@ -51,18 +52,42 @@ usage(void)
 	fprintf(stderr, "  -r repo         The repository name\n");
 	fprintf(stderr, "  -p pr           PR id to comment under\n");
 	fprintf(stderr, "  -i issue        issue id to comment under\n");
+	fprintf(stderr, "  -R comment-id   Reply to the comment with the given ID\n");
 	fprintf(stderr, "  -y              Do not ask for confirmation\n");
 	version();
 	copyright();
 }
 
-static void
-comment_init(gcli_ctx *ctx, FILE *f, void *_data)
+struct submit_ctx {
+	struct gcli_submit_comment_opts opts;
+	struct gcli_comment reply_comment;
+};
+
+void
+gcli_print_prefixed(FILE *f, char const *const text, char const *const prefix)
 {
-	gcli_submit_comment_opts *info = _data;
+	char const *bol = text;
+
+	while (bol) {
+		char const *const eol = strchr(bol, '\n');
+		size_t const len = eol ? (size_t)(eol - bol) : strlen(bol);
+
+		fprintf(f, "%s%.*s\n", prefix, (int)len, bol);
+
+		if (!eol)
+			break;
+
+		bol = eol + 1;
+	}
+}
+
+static void
+comment_init(struct gcli_ctx *ctx, FILE *f, void *_data)
+{
+	struct submit_ctx *sctx = _data;
 	const char *target_type = NULL;
 
-	switch (info->target_type) {
+	switch (sctx->opts.target_type) {
 	case ISSUE_COMMENT:
 		target_type = "issue";
 		break;
@@ -75,102 +100,134 @@ comment_init(gcli_ctx *ctx, FILE *f, void *_data)
 		case GCLI_FORGE_GITLAB:
 			target_type = "Merge Request";
 			break;
+		case GCLI_FORGE_BUGZILLA:
+			/* FIXME think about this one */
+			assert(0 && "unreachable");
+			break;
 		}
 	} break;
 	}
 
+	/* In case we reply to a comment, put the comment prefixed with
+	 * '> ' into the file first. */
+	if (sctx->reply_comment.body)
+		gcli_print_prefixed(f, sctx->reply_comment.body, "> ");
+
 	fprintf(
 		f,
 		"! Enter your comment above, save and exit.\n"
 		"! All lines with a leading '!' are discarded and will not\n"
 		"! appear in your comment.\n"
-		"! COMMENT IN : %s/%s %s #%d\n",
-		info->owner, info->repo, target_type, info->target_id);
+		"!\n"
+		"! vim: ft=markdown\n");
+
+	/* XXX */
+	if (sctx->opts.target.kind == GCLI_PATH_DEFAULT) {
+		fprintf(f, "! COMMENT IN : %s/%s %s #%"PRIid"\n",
+		        sctx->opts.target.as_default.owner,
+		        sctx->opts.target.as_default.repo, target_type,
+		        sctx->opts.target.as_default.id);
+	}
 }
 
-static sn_sv
-gcli_comment_get_message(gcli_submit_comment_opts *info)
+static char *
+gcli_comment_get_message(struct submit_ctx *info)
 {
 	return gcli_editor_get_user_message(g_clictx, comment_init, info);
 }
 
 static int
-comment_submit(gcli_submit_comment_opts opts, int always_yes)
+comment_submit(struct submit_ctx *sctx, int always_yes)
 {
-	sn_sv const message = gcli_comment_get_message(&opts);
-	opts.message = gcli_json_escape(message);
 	int rc = 0;
+	char *message;
 
-	fprintf(
-		stdout,
-		"You will be commenting the following in %s/%s #%d:\n"SV_FMT"\n",
-		opts.owner, opts.repo, opts.target_id, SV_ARGS(message));
+	message = gcli_comment_get_message(sctx);
+	sctx->opts.message = message;
+
+	if (message == NULL)
+		errx(1, "gcli: empty message. aborting.");
+
+	fprintf(stdout, "You will be commenting the following:\n");
+	gcli_pretty_print(sctx->opts.message, 4, 80, stdout);
 
 	if (!always_yes) {
-		if (!sn_yesno("Is this okay?"))
+		if (!gcli_yesno("Is this okay?"))
 			errx(1, "Aborted by user");
 	}
 
-	rc = gcli_comment_submit(g_clictx, opts);
+	rc = gcli_comment_submit(g_clictx, &sctx->opts);
 
-	free(message.data);
-	free(opts.message.data);
+	free(message);
+	sctx->opts.message = NULL;
 
 	return rc;
 }
 
 int
-gcli_issue_comments(char const *owner, char const *repo, int const issue)
+gcli_issue_comments(struct gcli_path const *const path)
 {
-	gcli_comment_list list = {0};
+	struct gcli_comment_list list = {0};
 	int rc = 0;
 
-	rc = gcli_get_issue_comments(g_clictx, owner, repo, issue, &list);
+	rc = gcli_get_issue_comments(g_clictx, path, &list);
 	if (rc < 0)
 		return rc;
 
 	gcli_print_comment_list(&list);
-	gcli_comment_list_free(&list);
+	gcli_comments_free(&list);
 
 	return rc;
 }
 
 int
-gcli_pull_comments(char const *owner, char const *repo, int const pull)
+gcli_pull_comments(struct gcli_path const *const pull_path)
 {
-	gcli_comment_list list = {0};
+	struct gcli_comment_list list = {0};
 	int rc = 0;
 
-	rc = gcli_get_pull_comments(g_clictx, owner, repo, pull, &list);
+	rc = gcli_get_pull_comments(g_clictx, pull_path, &list);
 	if (rc < 0)
 		return rc;
 
 	gcli_print_comment_list(&list);
-	gcli_comment_list_free(&list);
+	gcli_comments_free(&list);
 
 	return rc;
 }
 
 void
-gcli_print_comment_list(gcli_comment_list const *const list)
+gcli_print_comment_list(struct gcli_comment_list const *const list)
 {
 	for (size_t i = 0; i < list->comments_size; ++i) {
+		int rc = 0;
+		char *date = NULL;
+
+		rc = gcli_format_as_localtime(g_clictx, list->comments[i].date,
+		                              &date);
+		if (rc < 0)
+			err(1, "gcli: error: couldn't format timestamp");
+
 		printf("AUTHOR : %s%s%s\n"
-		       "DATE   : %s\n",
+		       "DATE   : %s\n"
+		       "ID     : %"PRIid"\n",
 		       gcli_setbold(), list->comments[i].author, gcli_resetbold(),
-		       list->comments[i].date);
-		pretty_print(list->comments[i].body, 9, 80, stdout);
+		       date,
+		       list->comments[i].id);
+		gcli_pretty_print(list->comments[i].body, 9, 80, stdout);
 		putchar('\n');
+
+		free(date);
 	}
 }
 
 int
 subcommand_comment(int argc, char *argv[])
 {
-	int ch, target_id = -1, rc = 0;
-	char const *repo = NULL, *owner = NULL;
+	int ch, rc = 0;
+	struct submit_ctx sctx = {0};
 	bool always_yes = false;
-	enum comment_target_type target_type;
+	gcli_id reply_to_id = 0;
 
 	struct option const options[] = {
 		{ .name    = "yes",
@@ -193,31 +250,41 @@ subcommand_comment(int argc, char *argv[])
 		  .has_arg = required_argument,
 		  .flag    = NULL,
 		  .val     = 'p' },
+		{ .name    = "in-reply-to",
+		  .has_arg = required_argument,
+		  .flag    = NULL,
+		  .val     = 'R' },
 		{0},
 	};
 
-	while ((ch = getopt_long(argc, argv, "yr:o:i:p:", options, NULL)) != -1) {
+	while ((ch = getopt_long(argc, argv, "yr:o:i:p:R:", options, NULL)) != -1) {
 		switch (ch) {
 		case 'r':
-			repo = optarg;
+			sctx.opts.target.as_default.repo = optarg;
 			break;
 		case 'o':
-			owner = optarg;
+			sctx.opts.target.as_default.owner = optarg;
 			break;
 		case 'p':
-			target_type = PR_COMMENT;
+			sctx.opts.target_type = PR_COMMENT;
 			goto parse_target_id;
 		case 'i':
-			target_type = ISSUE_COMMENT;
+			sctx.opts.target_type = ISSUE_COMMENT;
 		parse_target_id: {
 				char *endptr;
-				target_id = strtoul(optarg, &endptr, 10);
+				sctx.opts.target.as_default.id = strtoul(optarg, &endptr, 10);
 				if (endptr != optarg + strlen(optarg))
-					err(1, "error: Cannot parse issue/PR number");
+					err(1, "gcli: error: Cannot parse issue/PR number");
 			} break;
 		case 'y':
 			always_yes = true;
 			break;
+		case 'R': {
+			char *endptr = NULL;
+			reply_to_id = strtoul(optarg, &endptr, 10);
+			if (endptr != optarg + strlen(optarg))
+				err(1, "gcli: error: cannot parse comment id");
+		} break;
 		default:
 			usage();
 			return EXIT_FAILURE;
@@ -227,23 +294,31 @@ subcommand_comment(int argc, char *argv[])
 	argc -= optind;
 	argv += optind;
 
-	check_owner_and_repo(&owner, &repo);
+	check_path(&sctx.opts.target);
 
-	if (target_id < 0) {
-		fprintf(stderr, "error: missing issue/PR number (use -i/-p)\n");
+	if (!sctx.opts.target.as_default.id) {
+		fprintf(stderr, "gcli: error: missing issue/PR number (use -i/-p)\n");
 		usage();
 		return EXIT_FAILURE;
 	}
 
-	rc = comment_submit((gcli_submit_comment_opts) {
-			.owner = owner,
-			.repo = repo,
-			.target_type = target_type,
-			.target_id = target_id },
-		always_yes);
+	if (reply_to_id) {
+		rc = gcli_get_comment(g_clictx, &sctx.opts.target,
+		                      sctx.opts.target_type, reply_to_id,
+		                      &sctx.reply_comment);
+
+		if (rc < 0) {
+			errx(1, "gcli: error: failed to fetch comment for reply: %s",
+			     gcli_get_error(g_clictx));
+		}
+	}
+
+	rc = comment_submit(&sctx, always_yes);
+
+	gcli_comment_free(&sctx.reply_comment);
 
 	if (rc < 0)
-		errx(1, "error: failed to submit comment: %s", gcli_get_error(g_clictx));
+		errx(1, "gcli: error: failed to submit comment: %s", gcli_get_error(g_clictx));
 
 	return EXIT_SUCCESS;
 }
