@@ -1,5 +1,5 @@
 /*
- * Copyright 2023 Nico Sonack <nsonack@herrhotzenplotz.de>
+ * Copyright 2023-2025 Nico Sonack <nsonack@herrhotzenplotz.de>
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -29,12 +29,15 @@
 
 #include <config.h>
 
+#include <gcli/cmd/actions.h>
 #include <gcli/cmd/cmd.h>
 #include <gcli/cmd/issues.h>
+#include <gcli/cmd/open.h>
 #include <gcli/cmd/table.h>
 
 #include <gcli/forges.h>
 #include <gcli/milestones.h>
+#include <gcli/port/util.h>
 
 #ifdef HAVE_GETOPT_H
 #include <getopt.h>
@@ -58,20 +61,21 @@ usage(void)
 	fprintf(stderr, "  issues               List issues associated with the milestone\n");
 	fprintf(stderr, "  set-duedate <date>   Set due date \n");
 	fprintf(stderr, "  delete               Delete this milestone\n");
+	fprintf(stderr, "  open                 Open this milestone in a web browser\n");
 	fprintf(stderr, "\n");
 	version();
 	copyright();
 }
 
 void
-gcli_print_milestones(gcli_milestone_list const *const list, int max)
+gcli_print_milestones(struct gcli_milestone_list const *const list, int max)
 {
 	size_t n;
 	gcli_tbl tbl;
-	gcli_tblcoldef cols[] = {
-		{ .name = "ID",      .type = GCLI_TBLCOLTYPE_INT,    .flags = GCLI_TBLCOL_JUSTIFYR },
+	struct gcli_tblcoldef cols[] = {
+		{ .name = "ID",      .type = GCLI_TBLCOLTYPE_ID,     .flags = GCLI_TBLCOL_JUSTIFYR },
 		{ .name = "STATE",   .type = GCLI_TBLCOLTYPE_STRING, .flags = GCLI_TBLCOL_STATECOLOURED },
-		{ .name = "CREATED", .type = GCLI_TBLCOLTYPE_STRING, .flags = 0 },
+		{ .name = "CREATED", .type = GCLI_TBLCOLTYPE_TIME_T, .flags = 0 },
 		{ .name = "TITLE",   .type = GCLI_TBLCOLTYPE_STRING, .flags = 0 },
 	};
 
@@ -82,7 +86,7 @@ gcli_print_milestones(gcli_milestone_list const *const list, int max)
 
 	tbl = gcli_tbl_begin(cols, ARRAY_SIZE(cols));
 	if (!tbl)
-		errx(1, "error: could not init table printer");
+		errx(1, "gcli: error: could not init table printer");
 
 	if (max < 0 || (size_t)(max) > list->milestones_size)
 		n = list->milestones_size;
@@ -101,23 +105,23 @@ gcli_print_milestones(gcli_milestone_list const *const list, int max)
 }
 
 void
-gcli_print_milestone(gcli_milestone const *const milestone)
+gcli_print_milestone(struct gcli_milestone const *const milestone)
 {
 	gcli_dict dict;
 	uint32_t const quirks = gcli_forge(g_clictx)->milestone_quirks;
 
 	dict = gcli_dict_begin();
-	gcli_dict_add(dict,        "ID", 0, 0, "%d", milestone->id);
-	gcli_dict_add_string(dict, "TITLE", 0, 0, milestone->title);
-	gcli_dict_add_string(dict, "STATE", GCLI_TBLCOL_STATECOLOURED, 0, milestone->state);
-	gcli_dict_add_string(dict, "CREATED", 0, 0, milestone->created_at);
-	gcli_dict_add_string(dict, "UPDATED", 0, 0, milestone->created_at);
+	gcli_dict_add(dict,           "ID", 0, 0, "%"PRIid, milestone->id);
+	gcli_dict_add_string(dict,    "TITLE", 0, 0, milestone->title);
+	gcli_dict_add_string(dict,    "STATE", GCLI_TBLCOL_STATECOLOURED, 0, milestone->state);
+	gcli_dict_add_timestamp(dict, "CREATED", 0, 0, milestone->created_at);
+	gcli_dict_add_timestamp(dict, "UPDATED", 0, 0, milestone->created_at);
 
 	if ((quirks & GCLI_MILESTONE_QUIRKS_DUEDATE) == 0)
-		gcli_dict_add_string(dict, "DUE", 0, 0, milestone->due_date);
+		gcli_dict_add_timestamp(dict, "DUE", 0, 0, milestone->due_date);
 
 	if ((quirks & GCLI_MILESTONE_QUIRKS_EXPIRED) == 0)
-		gcli_dict_add_string(dict, "EXPIRED", 0, 0, sn_bool_yesno(milestone->expired));
+		gcli_dict_add_string(dict, "EXPIRED", 0, 0, gcli_bool_yesno(milestone->expired));
 
 	if ((quirks & GCLI_MILESTONE_QUIRKS_NISSUES) == 0) {
 		gcli_dict_add(dict, "OPEN ISSUES", 0, 0, "%d", milestone->open_issues);
@@ -128,21 +132,216 @@ gcli_print_milestone(gcli_milestone const *const milestone)
 
 	if (milestone->description && strlen(milestone->description)) {
 		printf("\nDESCRIPTION:\n");
-		pretty_print(milestone->description, 4, 80, stdout);
+		gcli_pretty_print(milestone->description, 4, 80, stdout);
 	}
 }
 
-static int handle_milestone_actions(int argc, char *argv[],
-                                    char const *const owner,
-                                    char const *const repo,
-                                    int const milestone_id);
+static int
+action_milestone_all(struct gcli_path const *const path,
+                     struct gcli_milestone const *item,
+                     int *argc, char **argv[])
+{
+	struct gcli_issue_list issues = {0};
+	int rc = 0;
 
+	(void) argc;
+	(void) argv;
+
+	gcli_print_milestone(item);
+	rc = gcli_milestone_get_issues(g_clictx, path, &issues);
+
+	if (rc < 0) {
+		fprintf(stderr, "gcli: error: failed to fetch issues: %s\n",
+		        gcli_get_error(g_clictx));
+
+		return GCLI_EX_DATAERR;
+	}
+
+	printf("\nISSUES:\n");
+	gcli_print_issues(0, &issues, -1);
+	gcli_issues_free(&issues);
+
+	return GCLI_EX_OK;
+}
+
+static int
+action_milestone_issues(struct gcli_path const *const path,
+                        struct gcli_milestone const *item,
+                        int *argc, char **argv[])
+{
+	int rc = 0;
+	struct gcli_issue_list issues = {0};
+
+	(void) item;
+	(void) argc;
+	(void) argv;
+
+	/* Fetch list of issues associated with milestone */
+	rc = gcli_milestone_get_issues(g_clictx, path, &issues);
+	if (rc < 0) {
+		fprintf(stderr, "gcli: error: failed to get issues: %s\n",
+		        gcli_get_error(g_clictx));
+
+		return GCLI_EX_DATAERR;
+	}
+
+	/* Print them as a table */
+	gcli_print_issues(0, &issues, -1);
+
+	/* Cleanup */
+	gcli_issues_free(&issues);
+
+	return GCLI_EX_OK;
+}
+
+static int
+action_milestone_status(struct gcli_path const *const path,
+                        struct gcli_milestone const *const milestone,
+                        int *argc, char **argv[])
+{
+	(void) path;
+	(void) argc;
+	(void) argv;
+
+	gcli_print_milestone(milestone);
+	return GCLI_EX_OK;
+}
+
+static int
+action_milestone_delete(struct gcli_path const *const path,
+                        struct gcli_milestone const *const milestone,
+                        int *argc, char **argv[])
+{
+	int rc;
+
+	(void) milestone;
+	(void) argc;
+	(void) argv;
+
+	/* Delete the milestone */
+	rc = gcli_delete_milestone(g_clictx, path);
+	if (rc < 0) {
+		fprintf(stderr, "gcli: error: could not delete milestone: %s\n",
+		        gcli_get_error(g_clictx));
+
+		return GCLI_EX_DATAERR;
+	}
+
+	return GCLI_EX_OK;
+}
+
+static int
+action_milestone_set_duedate(struct gcli_path const *const path,
+                             struct gcli_milestone const *const milestone,
+                             int *argc, char **argv[])
+{
+	char *new_date = NULL;
+	int rc = 0;
+
+	(void) milestone;
+
+	/* grab the the date that the user provided */
+	if (*argc < 2) {
+		fprintf(stderr, "gcli: error: missing date for set-duedate\n");
+		return GCLI_EX_USAGE;
+	}
+
+	shift(argc, argv);
+	new_date = (*argv)[0];
+
+	/* Do it! */
+	rc = gcli_milestone_set_duedate(g_clictx, path, new_date);
+	if (rc < 0) {
+		fprintf(stderr, "gcli: error: could not update milestone due date: %s\n",
+		        gcli_get_error(g_clictx));
+
+		return GCLI_EX_DATAERR;
+	}
+
+	return GCLI_EX_OK;
+}
+
+static int
+action_milestone_open(struct gcli_path const *const path,
+                      struct gcli_milestone const *const milestone,
+                      int *argc, char **argv[])
+{
+	int rc;
+
+	(void) path;
+	(void) argc;
+	(void) argv;
+
+	rc = gcli_cmd_open_url(milestone->web_url);
+	if (rc < 0) {
+		fprintf(stderr, "gcli: error: failed to open url\n");
+		return GCLI_EX_DATAERR;
+	}
+
+	return GCLI_EX_OK;
+}
+
+struct gcli_cmd_actions milestone_actions = {
+	.fetch_item = (gcli_cmd_action_fetcher)gcli_get_milestone,
+	.free_item = (gcli_cmd_action_freeer)gcli_free_milestone,
+	.item_size = sizeof(struct gcli_milestone),
+
+	.defs = {
+		{
+			.name = "all",
+			.needs_item = true,
+			.handler = (gcli_cmd_action_handler)action_milestone_all,
+		},
+		{
+			.name = "issues",
+			.needs_item = false,
+			.handler = (gcli_cmd_action_handler)action_milestone_issues,
+		},
+		{
+			.name = "status",
+			.needs_item = true,
+			.handler = (gcli_cmd_action_handler)action_milestone_status,
+		},
+		{
+			.name = "delete",
+			.needs_item = false,
+			.handler = (gcli_cmd_action_handler)action_milestone_delete,
+		},
+		{
+			.name = "set-duedate",
+			.needs_item = false,
+			.handler = (gcli_cmd_action_handler)action_milestone_set_duedate,
+		},
+		{
+			.name = "open",
+			.needs_item = true,
+			.handler = (gcli_cmd_action_handler)action_milestone_open,
+		},
+		{0},
+	},
+};
+
+static int
+handle_milestone_actions(int argc, char *argv[],
+                         struct gcli_path const *const path)
+{
+	int rc = 0;
+
+	rc = gcli_cmd_actions_handle(&milestone_actions, path, &argc, &argv);
+	if (rc == GCLI_EX_USAGE) {
+		usage();
+	}
+
+	return rc ? EXIT_FAILURE : EXIT_SUCCESS;
+}
 
 static int
 subcommand_milestone_create(int argc, char *argv[])
 {
 	int ch;
 	struct gcli_milestone_create_args args = {0};
+	struct gcli_path repo = {0};
+
 	struct option const options[] = {
 		{ .name = "owner",
 		  .has_arg = required_argument,
@@ -166,10 +365,10 @@ subcommand_milestone_create(int argc, char *argv[])
 	while ((ch = getopt_long(argc, argv, "+o:r:t:d:", options, NULL)) != -1) {
 		switch (ch) {
 		case 'o':
-			args.owner = optarg;
+			repo.as_default.owner = optarg;
 			break;
 		case 'r':
-			args.repo = optarg;
+			repo.as_default.repo = optarg;
 			break;
 		case 't':
 			args.title = optarg;
@@ -188,18 +387,18 @@ subcommand_milestone_create(int argc, char *argv[])
 
 	/* sanity check argumets */
 	if (argc)
-		errx(1, "error: stray arguments");
+		errx(1, "gcli: error: stray arguments");
 
-	/* make sure both are set or deduce them */
-	check_owner_and_repo(&args.owner, &args.repo);
+	/* make sure path is valid */
+	check_path(&repo);
 
 	/* enforce the user to at least provide a title */
 	if (!args.title)
-		errx(1, "error: missing milestone title");
+		errx(1, "gcli: error: missing milestone title");
 
 	/* actually create the milestone */
-	if (gcli_create_milestone(g_clictx, &args) < 0)
-		errx(1, "error: could not create milestone: %s",
+	if (gcli_create_milestone(g_clictx, &repo, &args) < 0)
+		errx(1, "gcli: error: could not create milestone: %s",
 		     gcli_get_error(g_clictx));
 
 	return 0;
@@ -208,8 +407,8 @@ subcommand_milestone_create(int argc, char *argv[])
 int
 subcommand_milestones(int argc, char *argv[])
 {
-	int ch, rc, max = 30, milestone_id = -1;
-	char const *repo, *owner;
+	int ch, rc, max = 30;
+	struct gcli_path path = {0};
 	struct option const options[] = {
 		{ .name = "owner",
 		  .has_arg = required_argument,
@@ -236,31 +435,25 @@ subcommand_milestones(int argc, char *argv[])
 	}
 
 	/* Proceed with fetching information */
-	repo = NULL;
-	owner = NULL;
-
 	while ((ch = getopt_long(argc, argv, "+o:r:n:i:", options, NULL)) != -1) {
 		switch (ch) {
 		case 'o': {
-			owner = optarg;
+			path.as_default.owner = optarg;
 		} break;
 		case 'r': {
-			repo = optarg;
+			path.as_default.repo = optarg;
 		} break;
 		case 'n': {
 			char *endptr;
 			max = strtol(optarg, &endptr, 10);
 			if (endptr != optarg + strlen(optarg))
-				errx(1, "error: cannot parse milestone count");
+				errx(1, "gcli: error: cannot parse milestone count");
 		} break;
 		case 'i': {
 			char *endptr;
-			milestone_id = strtol(optarg, &endptr, 10);
+			path.as_default.id = strtoul(optarg, &endptr, 10);
 			if (endptr != optarg + strlen(optarg))
-				errx(1, "error: cannot parse milestone id");
-
-			if (milestone_id < 0)
-				errx(1, "error: milestone id must not be negative");
+				errx(1, "gcli: error: cannot parse milestone id");
 		} break;
 		default: {
 			usage();
@@ -272,15 +465,16 @@ subcommand_milestones(int argc, char *argv[])
 	argc -= optind;
 	argv += optind;
 
-	check_owner_and_repo(&owner, &repo);
+	check_path(&path);
 
-	if (milestone_id < 0) {
-		gcli_milestone_list list = {0};
+	if (path.as_default.id == 0) {
+		struct gcli_milestone_list list = {0};
 
-		rc = gcli_get_milestones(g_clictx, owner, repo, max, &list);
-		if (rc < 0)
-			errx(1, "error: cannot get list of milestones: %s",
+		rc = gcli_get_milestones(g_clictx, &path, max, &list);
+		if (rc < 0) {
+			errx(1, "gcli: error: cannot get list of milestones: %s",
 			     gcli_get_error(g_clictx));
+		}
 
 		gcli_print_milestones(&list, max);
 
@@ -289,140 +483,5 @@ subcommand_milestones(int argc, char *argv[])
 		return 0;
 	}
 
-	return handle_milestone_actions(argc, argv, owner, repo, milestone_id);
-}
-
-static void
-ensure_milestone(char const *const owner,
-                 char const *const repo,
-                 int const milestone_id,
-                 int *const fetched_milestone,
-                 gcli_milestone *const milestone)
-{
-	int rc;
-
-	if (*fetched_milestone)
-		return;
-
-	rc = gcli_get_milestone(g_clictx, owner, repo, milestone_id, milestone);
-	if (rc < 0)
-		errx(1, "error: could not get milestone %d: %s", milestone_id,
-		     gcli_get_error(g_clictx));
-
-	*fetched_milestone = 1;
-}
-
-static int
-handle_milestone_actions(int argc, char *argv[],
-                         char const *const owner,
-                         char const *const repo,
-                         int const milestone_id)
-{
-	gcli_milestone milestone = {0};
-	int fetched_milestone = 0;
-
-	/* Check if the user missed out on supplying actions */
-	if (argc == 0) {
-		fprintf(stderr, "error: no actions supplied\n");
-		usage();
-		exit(EXIT_FAILURE);
-	}
-
-
-	/* Iterate over all the actions */
-	while (argc) {
-		/* Read in action */
-		char const *action = shift(&argc, &argv);
-
-		/* Dispatch */
-		if (strcmp(action, "all") == 0) {
-			int rc = 0;
-
-			gcli_issue_list issues = {0};
-
-			ensure_milestone(owner, repo, milestone_id,
-			                 &fetched_milestone, &milestone);
-
-			gcli_print_milestone(&milestone);
-			rc = gcli_milestone_get_issues(g_clictx, owner, repo, milestone_id,
-			                               &issues);
-
-			if (rc < 0)
-				errx(1, "error: failed to fetch issues: %s",
-				     gcli_get_error(g_clictx));
-
-			printf("\nISSUES:\n");
-			gcli_print_issues(0, &issues, -1);
-			gcli_issues_free(&issues);
-
-		} else if (strcmp(action, "issues") == 0) {
-			int rc = 0;
-			gcli_issue_list issues = {0};
-
-			/* Fetch list of issues associated with milestone */
-			rc = gcli_milestone_get_issues(g_clictx, owner, repo, milestone_id,
-			                               &issues);
-			if (rc < 0)
-				errx(1, "error: failed to fetch issues: %s",
-				     gcli_get_error(g_clictx));
-
-			/* Print them as a table */
-			gcli_print_issues(0, &issues, -1);
-
-			/* Cleanup */
-			gcli_issues_free(&issues);
-
-		} else if (strcmp(action, "status") == 0) {
-
-			/* Make sure we have the milestone data */
-			ensure_milestone(owner, repo, milestone_id,
-			                 &fetched_milestone, &milestone);
-
-			/* Print meta */
-			gcli_print_milestone(&milestone);
-		} else if (strcmp(action, "delete") == 0) {
-
-			/* Delete the milestone */
-			if (gcli_delete_milestone(g_clictx, owner, repo, milestone_id) < 0)
-				errx(1, "error: could not delete milestone: %s",
-				     gcli_get_error(g_clictx));
-
-		} else if (strcmp(action, "set-duedate") == 0) {
-
-			char *new_date = NULL;
-			int rc = 0;
-
-			/* grab the the date that the user provided */
-			if (!argc)
-				errx(1, "error: missing date for set-duedate");
-
-			new_date = shift(&argc, &argv);
-
-			/* Do it! */
-			rc = gcli_milestone_set_duedate(g_clictx, owner, repo, milestone_id,
-			                                new_date);
-			if (rc < 0)
-				errx(1, "error: could not update milestone due date: %s",
-				     gcli_get_error(g_clictx));;
-
-		} else {
-
-			/* We don't know of the action - maybe a syntax error or
-			 * trailing garbage. Error out in this case. */
-			fprintf(stderr, "error: unknown action %s\n", action);
-			usage();
-			return EXIT_FAILURE;
-		}
-
-
-		/* Print a blank line if we are not at the end */
-		if (argc)
-			putchar('\n');
-	}
-
-	/* Cleanup the milestone if we ever fetched it */
-	if (fetched_milestone)
-		gcli_free_milestone(&milestone);
-
-	return EXIT_SUCCESS;
+	return handle_milestone_actions(argc, argv, &path);
 }
