@@ -1,5 +1,5 @@
 /*
- * Copyright 2022 Nico Sonack <nsonack@herrhotzenplotz.de>
+ * Copyright 2022-2025 Nico Sonack <nsonack@herrhotzenplotz.de>
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -34,6 +34,7 @@
 #include <gcli/cmd/repos.h>
 #include <gcli/cmd/table.h>
 
+#include <gcli/port/util.h>
 #include <gcli/repos.h>
 
 #ifdef HAVE_GETOPT_H
@@ -46,14 +47,20 @@ static void
 usage(void)
 {
 	fprintf(stderr, "usage: gcli repos create -r repo [-d description] [-p]\n");
-	fprintf(stderr, "       gcli repos [-o owner -r repo] [-n number] [-s] [-y] [delete]\n");
+	fprintf(stderr, "       gcli repos [-o owner -r repo] [-n number] [-s]\n");
+	fprintf(stderr, "       gcli repos [-o owner -r repo] actions...\n");
 	fprintf(stderr, "OPTIONS:\n");
-	fprintf(stderr, "  -o owner        The repository owner\n");
-	fprintf(stderr, "  -r repo         The repository name\n");
-	fprintf(stderr, "  -n number       Number of repos to fetch (-1 = everything)\n");
-	fprintf(stderr, "  -p              Make the repo private\n");
-	fprintf(stderr, "  -s              Print (sort) in reverse order\n");
-	fprintf(stderr, "  -y              Do not ask for confirmation\n");
+	fprintf(stderr, "  -o owner                The repository owner\n");
+	fprintf(stderr, "  -r repo                 The repository name\n");
+	fprintf(stderr, "  -n number               Number of repos to fetch (-1 = everything)\n");
+	fprintf(stderr, "  -p                      Make the repo private\n");
+	fprintf(stderr, "  -s                      Print (sort) in reverse order\n");
+	fprintf(stderr, "ACTIONS:\n");
+	fprintf(stderr, "  delete [-y]             Delete this repository:\n");
+	fprintf(stderr, "                            -y    Do not ask for confirmation\n");
+	fprintf(stderr, "  set-visibility <level>  Mark the reposity as public or private. Level may be one of:\n");
+	fprintf(stderr, "                            - public\n");
+	fprintf(stderr, "                            - private\n");
 	fprintf(stderr, "\n");
 	version();
 	copyright();
@@ -61,15 +68,15 @@ usage(void)
 
 void
 gcli_print_repos(enum gcli_output_flags const flags,
-                 gcli_repo_list const *const list, int const max)
+                 struct gcli_repo_list const *const list, int const max)
 {
 	size_t n;
 	gcli_tbl table;
-	gcli_tblcoldef cols[] = {
-		{ .name = "FORK",     .type = GCLI_TBLCOLTYPE_BOOL, .flags = 0 },
-		{ .name = "VISBLTY",  .type = GCLI_TBLCOLTYPE_SV,   .flags = 0 },
-		{ .name = "DATE",     .type = GCLI_TBLCOLTYPE_SV,   .flags = 0 },
-		{ .name = "FULLNAME", .type = GCLI_TBLCOLTYPE_SV,   .flags = 0 },
+	struct gcli_tblcoldef cols[] = {
+		{ .name = "FORK",     .type = GCLI_TBLCOLTYPE_BOOL,   .flags = 0 },
+		{ .name = "VISBLTY",  .type = GCLI_TBLCOLTYPE_STRING, .flags = 0 },
+		{ .name = "DATE",     .type = GCLI_TBLCOLTYPE_TIME_T, .flags = 0 },
+		{ .name = "FULLNAME", .type = GCLI_TBLCOLTYPE_STRING, .flags = 0 },
 	};
 
 	if (list->repos_size == 0) {
@@ -86,7 +93,7 @@ gcli_print_repos(enum gcli_output_flags const flags,
 	/* init table */
 	table = gcli_tbl_begin(cols, ARRAY_SIZE(cols));
 	if (!table)
-		errx(1, "error: could not init table");
+		errx(1, "gcli: error: could not init table");
 
 	/* put data into table */
 	if (flags & OUTPUT_SORTED) {
@@ -110,18 +117,18 @@ gcli_print_repos(enum gcli_output_flags const flags,
 }
 
 void
-gcli_repo_print(gcli_repo const *it)
+gcli_repo_print(struct gcli_repo const *it)
 {
 	gcli_dict dict;
 
 	dict = gcli_dict_begin();
-	gcli_dict_add(dict, "ID",         0, 0, "%d", it->id);
-	gcli_dict_add(dict, "FULL NAME",  0, 0, SV_FMT, SV_ARGS(it->full_name));
-	gcli_dict_add(dict, "NAME",       0, 0, SV_FMT, SV_ARGS(it->name));
-	gcli_dict_add(dict, "OWNER",      0, 0, SV_FMT, SV_ARGS(it->owner));
-	gcli_dict_add(dict, "DATE",       0, 0, SV_FMT, SV_ARGS(it->date));
-	gcli_dict_add(dict, "VISIBILITY", 0, 0, SV_FMT, SV_ARGS(it->visibility));
-	gcli_dict_add(dict, "IS FORK",    0, 0, "%s", sn_bool_yesno(it->is_fork));
+	gcli_dict_add(dict,           "ID",         0, 0, "%"PRIid, it->id);
+	gcli_dict_add(dict,           "FULL NAME",  0, 0, "%s", it->full_name);
+	gcli_dict_add(dict,           "NAME",       0, 0, "%s", it->name);
+	gcli_dict_add(dict,           "OWNER",      0, 0, "%s", it->owner);
+	gcli_dict_add_timestamp(dict, "DATE",       0, 0, it->date);
+	gcli_dict_add(dict,           "VISIBILITY", 0, 0, "%s", it->visibility);
+	gcli_dict_add(dict,           "IS FORK",    0, 0, "%s", gcli_bool_yesno(it->is_fork));
 
 	gcli_dict_end(dict);
 }
@@ -130,8 +137,8 @@ static int
 subcommand_repos_create(int argc, char *argv[])
 {
 	int ch;
-	gcli_repo_create_options create_options = {0};
-	gcli_repo repo = {0};
+	struct gcli_repo_create_options create_options = {0};
+	struct gcli_repo repo = {0};
 
 	const struct option options[] = {
 		{ .name    = "repo",
@@ -152,10 +159,10 @@ subcommand_repos_create(int argc, char *argv[])
 	while ((ch = getopt_long(argc, argv, "r:d:p", options, NULL)) != -1) {
 		switch (ch) {
 		case 'r':
-			create_options.name = SV(optarg);
+			create_options.name = optarg;
 			break;
 		case 'd':
-			create_options.description = SV(optarg);
+			create_options.description = optarg;
 			break;
 		case 'p':
 			create_options.private = true;
@@ -170,16 +177,18 @@ subcommand_repos_create(int argc, char *argv[])
 	argc -= optind;
 	argv += optind;
 
-	if (sn_sv_null(create_options.name)) {
+	if (!create_options.name) {
 		fprintf(stderr,
-		        "name cannot be empty. please set a repository "
+		        "gcli: name cannot be empty. please set a repository "
 		        "name with -r/--name\n");
 		usage();
 		return EXIT_FAILURE;
 	}
 
-	if (gcli_repo_create(g_clictx, &create_options, &repo) < 0)
-		errx(1, "error: failed to create repository: %s", gcli_get_error(g_clictx));
+	if (gcli_repo_create(g_clictx, &create_options, &repo) < 0) {
+		errx(1, "gcli: error: failed to create repository: %s",
+		     gcli_get_error(g_clictx));
+	}
 
 	gcli_repo_print(&repo);
 	gcli_repo_free(&repo);
@@ -187,15 +196,110 @@ subcommand_repos_create(int argc, char *argv[])
 	return EXIT_SUCCESS;
 }
 
+static int
+action_delete(struct gcli_path const *const path, int *argc, char ***argv)
+{
+	int ch;
+	bool always_yes = false;
+	struct option const options[] = {
+		{ .name    = "yes",
+		  .has_arg = no_argument,
+		  .flag    = NULL,
+		  .val     = 'y' },
+		{0},
+	};
+
+	while ((ch = getopt_long(*argc, *argv, "+y", options, NULL)) != -1) {
+		switch (ch) {
+		case 'y':
+			always_yes = true;
+			break;
+		default:
+			usage();
+			return EXIT_FAILURE;
+		}
+	}
+
+	*argc -= optind;
+	*argv += optind;
+
+	delete_repo(always_yes, path);
+
+	return 0;
+}
+
+static gcli_repo_visibility
+parse_visibility(char const *str)
+{
+	if (strcmp(str, "public") == 0) {
+		return GCLI_REPO_VISIBILITY_PUBLIC;
+	} else if (strcmp(str, "private") == 0) {
+		return GCLI_REPO_VISIBILITY_PRIVATE;
+	} else {
+		fprintf(stderr, "gcli: error: bad visibility level »%s«\n", str);
+		return 1;
+	}
+}
+
+/* Change the visibility level of a repository (e.g. public, private
+ * etc) */
+static int
+action_set_visibility(struct gcli_path const *const path, int *argc,
+                      char ***argv)
+{
+	char const *visblty_str;
+	gcli_repo_visibility visblty;
+	int rc;
+
+	if (*argc < 2) {
+		fprintf(stderr, "gcli: error: missing visibility level\n");
+		return 1;
+	}
+
+	visblty_str = (*argv)[1];
+	*argv += 2;
+	*argc -= 2;
+
+	visblty = parse_visibility(visblty_str);
+
+	if ((rc = gcli_repo_set_visibility(g_clictx, path, visblty)) < 0) {
+		fprintf(stderr, "gcli: error: failed to set visibility: %s\n",
+		        gcli_get_error(g_clictx));
+		return 1;
+	}
+
+	return 0;
+}
+
+static struct action {
+	char const *const name;
+	int (*fn)(struct gcli_path const *const path, int *argc, char ***argv);
+} const actions[] = {
+	{ .name = "delete",         .fn = action_delete },
+	{ .name = "set-visibility", .fn = action_set_visibility },
+};
+
+static size_t const actions_size = ARRAY_SIZE(actions);
+
+static struct action const *
+find_action(char const *const name)
+{
+	for (size_t i = 0; i < actions_size; ++i) {
+		if (strcmp(name, actions[i].name) == 0)
+			return &actions[i];
+	}
+
+	return NULL;
+}
+
 int
 subcommand_repos(int argc, char *argv[])
 {
-	int                     ch, n = 30;
-	char const             *owner      = NULL;
-	char const             *repo       = NULL;
-	gcli_repo_list          repos      = {0};
-	bool                    always_yes = false;
-	enum gcli_output_flags  flags      = 0;
+	int ch, n = 30;
+	char *owner = NULL;
+	char *repo = NULL;
+	struct gcli_repo_list repos = {0};
+	enum gcli_output_flags flags = 0;
 
 	/* detect whether we wanna create a repo */
 	if (argc > 1 && (strcmp(argv[1], "create") == 0)) {
@@ -216,10 +320,6 @@ subcommand_repos(int argc, char *argv[])
 		  .has_arg = required_argument,
 		  .flag    = NULL,
 		  .val     = 'o' },
-		{ .name    = "yes",
-		  .has_arg = no_argument,
-		  .flag    = NULL,
-		  .val     = 'y' },
 		{ .name    = "sorted",
 		  .has_arg = no_argument,
 		  .flag    = NULL,
@@ -227,7 +327,7 @@ subcommand_repos(int argc, char *argv[])
 		{0},
 	};
 
-	while ((ch = getopt_long(argc, argv, "n:o:r:ys", options, NULL)) != -1) {
+	while ((ch = getopt_long(argc, argv, "+n:o:r:s", options, NULL)) != -1) {
 		switch (ch) {
 		case 'o':
 			owner = optarg;
@@ -235,9 +335,6 @@ subcommand_repos(int argc, char *argv[])
 		case 'r':
 			repo = optarg;
 			break;
-		case 'y':
-			always_yes = true;
-			break;
 		case 's':
 			flags |= OUTPUT_SORTED;
 			break;
@@ -245,10 +342,10 @@ subcommand_repos(int argc, char *argv[])
 			char *endptr = NULL;
 			n = strtol(optarg, &endptr, 10);
 			if (endptr != (optarg + strlen(optarg)))
-				err(1, "repos: cannot parse repo count");
+				err(1, "gcli: error: cannot parse repo count");
 
 			if (n == 0)
-				errx(1, "error: number of repos must not be zero");
+				errx(1, "gcli: error: number of repos must not be zero");
 		} break;
 		case '?':
 		default:
@@ -259,13 +356,14 @@ subcommand_repos(int argc, char *argv[])
 
 	argc -= optind;
 	argv += optind;
+	optind = 0;
 
 	/* List repos of the owner */
 	if (argc == 0) {
 		int rc = 0;
 
 		if (repo) {
-			fprintf(stderr, "error: repos: no actions specified\n");
+			fprintf(stderr, "gcli: error: no actions specified\n");
 			usage();
 			return EXIT_FAILURE;
 		}
@@ -273,25 +371,46 @@ subcommand_repos(int argc, char *argv[])
 		if (!owner)
 			owner = gcli_config_get_account_name(g_clictx);
 
+		/* whenever there is no default account we would be passing NULL to
+		 * gcli_get_repos. This is bad since that causes segfaults down the
+		 * line. (https://github.com/herrhotzenplotz/gcli/issues/118) */
+		if (!owner) {
+			fprintf(stderr, "gcli: error: no account specified or no default"
+			        " account configured. use -o to provide an explicit"
+			        " account name.\n");
+			return EXIT_FAILURE;
+		}
+
 		rc = gcli_get_repos(g_clictx, owner, n, &repos);
-		if (rc < 0)
-			errx(1, "error: failed to fetch repos: %s", gcli_get_error(g_clictx));
+		if (rc < 0) {
+			errx(1, "gcli: error: failed to fetch repos: %s",
+			     gcli_get_error(g_clictx));
+		}
 
 		gcli_print_repos(flags, &repos, n);
 		gcli_repos_free(&repos);
 	} else {
-		check_owner_and_repo(&owner, &repo);
+		struct gcli_path path = {
+			.as_default = {
+				.owner = owner,
+				.repo = repo,
+			},
+		};
 
-		for (size_t i = 0; i < (size_t)argc; ++i) {
-			char const *action = argv[i];
+		check_path(&path);
 
-			if (strcmp(action, "delete") == 0) {
-				delete_repo(always_yes, owner, repo);
-			} else {
-				fprintf(stderr, "error: repos: unknown action '%s'\n", action);
-				usage();
+		while (argc) {
+			struct action const *action = find_action(argv[0]);
+			int rc = 0;
+
+			if (!action) {
+				fprintf(stderr, "gcli: error: unrecognised action »%s«\n", argv[0]);
 				return EXIT_FAILURE;
 			}
+
+			rc = action->fn(&path, &argc, &argv);
+			if (rc)
+				return rc;
 		}
 	}
 
