@@ -1,5 +1,5 @@
 /*
- * Copyright 2022 Nico Sonack <nsonack@herrhotzenplotz.de>
+ * Copyright 2022-2025 Nico Sonack <nsonack@herrhotzenplotz.de>
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -31,9 +31,12 @@
 
 #include <gcli/cmd/cmd.h>
 #include <gcli/cmd/status.h>
+#include <gcli/cmd/status_interactive.h>
 
 #include <gcli/status.h>
 
+#include <string.h>
+
 #ifdef HAVE_GETOPT_H
 #include <getopt.h>
 #endif
@@ -42,19 +45,21 @@ static void
 usage(void)
 {
 	fprintf(stderr, "usage: gcli status -m id\n");
-	fprintf(stderr, "       gcli status [-n number]\n");
+	fprintf(stderr, "       gcli status -l [-n number]\n");
+	fprintf(stderr, "       gcli status\n");
 	fprintf(stderr, "OPTIONS:\n");
-	fprintf(stderr, "  -n number       Number of messages to fetch\n");
-	fprintf(stderr, "  -m id           Mark the given message as read\n");
+	fprintf(stderr, "  -l          Print list of todo items and exit\n");
+	fprintf(stderr, "  -n number   Number of messages to fetch\n");
+	fprintf(stderr, "  -m id       Mark the given message as read\n");
 	fprintf(stderr, "\n");
 	version();
 	copyright();
 }
 
 int
-gcli_status(int const count)
+gcli_status_list(int const count)
 {
-	gcli_notification_list list = {0};
+	struct gcli_notification_list list = {0};
 	int rc = 0;
 
 	rc = gcli_get_notifications(g_clictx, count, &list);
@@ -68,16 +73,22 @@ gcli_status(int const count)
 }
 
 void
-gcli_print_notifications(gcli_notification_list const *const list)
+gcli_print_notifications(struct gcli_notification_list const *const list)
 {
 	for (size_t i = 0; i < list->notifications_size; ++i) {
-		printf("%s - %s - %s - %s - %s\n",
+		printf("%s - %s - %s",
 		       list->notifications[i].id,
 		       list->notifications[i].repository,
-		       list->notifications[i].type, list->notifications[i].date,
-		       list->notifications[i].reason);
+		       gcli_notification_target_type_str(list->notifications[i].type));
 
-		pretty_print(list->notifications[i].title, 4, 80, stdout);
+		printf(" - %s", list->notifications[i].date);
+
+		if (list->notifications[i].reason)
+			printf(" - %s", list->notifications[i].reason);
+
+		printf("\n");
+
+		gcli_pretty_print(list->notifications[i].title, 4, 80, stdout);
 		putchar('\n');
 	}
 }
@@ -85,10 +96,8 @@ gcli_print_notifications(gcli_notification_list const *const list)
 int
 subcommand_status(int argc, char *argv[])
 {
-	int   count  = 30;
-	int   ch     = 0;
+	int count = 30, ch = 0, mark = 0, list = 0;
 	char *endptr = NULL;
-	int   mark   = 0;
 
 	const struct option options[] = {
 		{ .name    = "count",
@@ -99,19 +108,26 @@ subcommand_status(int argc, char *argv[])
 		  .has_arg = no_argument,
 		  .flag    = &mark,
 		  .val     = 1 },
+		{ .name    = "list",
+		  .has_arg = no_argument,
+		  .flag    = &list,
+		  .val     = 1 },
 		{0}
 	};
 
-	while ((ch = getopt_long(argc, argv, "n:m", options, NULL)) != -1) {
+	while ((ch = getopt_long(argc, argv, "n:ml", options, NULL)) != -1) {
 		switch (ch) {
 		case 'n': {
 			count = strtol(optarg, &endptr, 10);
 			if (endptr != optarg + strlen(optarg))
-				err(1, "status: cannot parse parameter to -n");
+				err(1, "gcli: error: cannot parse parameter to -n");
 		} break;
 		case 'm': {
 			mark = 1;
 		} break;
+		case 'l': {
+			list = 1;
+		} break;
 		default:
 			usage();
 			return EXIT_FAILURE;
@@ -121,27 +137,36 @@ subcommand_status(int argc, char *argv[])
 	argc -= optind;
 	argv += optind;
 
-	if (!mark) {
-		gcli_status(count);
-	} else {
+	if (list) {
+		if (mark) {
+			fprintf(stderr, "gcli: cannot use -m and -l together\n");
+			usage();
+			return EXIT_FAILURE;
+		}
+
+		gcli_status_list(count);
+
+	} else if (mark) {
 		if (count != 30)
-			warnx("ignoring -n/--count argument");
+			gcli_warnx(g_clictx, "gcli: ignoring -n/--count argument");
 
 		if (argc > 1) {
-			fprintf(stderr, "error: too many arguments for marking notifications\n");
+			fprintf(stderr, "gcli: error: too many arguments for marking notifications\n");
 			usage();
 			return EXIT_FAILURE;
 		}
 
 		if (argc < 1) {
-			fprintf(stderr, "error: missing notification id to mark as read\n");
+			fprintf(stderr, "gcli: error: missing notification id to mark as read\n");
 			usage();
 			return EXIT_FAILURE;
 		}
 
 		if (gcli_notification_mark_as_read(g_clictx, argv[0]) < 0)
-			errx(1, "error: failed to mark the notification as read: %s",
+			errx(1, "gcli: error: failed to mark the notification as read: %s",
 			     gcli_get_error(g_clictx));
+	} else {
+		return gcli_status_interactive();
 	}
 
 	return EXIT_SUCCESS;
