@@ -1,5 +1,5 @@
 /*
- * Copyright 2021, 2022 Nico Sonack <nsonack@herrhotzenplotz.de>
+ * Copyright 2021-2025 Nico Sonack <nsonack@herrhotzenplotz.de>
  * Copyright 2022 Aritra Sarkar <aritra1911@yahoo.com>
  *
  * Redistribution and use in source and binary forms, with or without
@@ -34,9 +34,9 @@
 #include <gcli/curl.h>
 #include <gcli/forges.h>
 #include <gcli/json_util.h>
+#include <gcli/port/err.h>
 
 #include <curl/curl.h>
-#include <sn/sn.h>
 #include <pdjson/pdjson.h>
 
 /* Hack for NetBSD's and Oracle Solaris broken isalnum implementation */
@@ -60,17 +60,20 @@ gcli_curl_isalnum(char const c)
 
 /* XXX move to gcli_ctx destructor */
 void
-gcli_curl_ctx_destroy(gcli_ctx *ctx)
+gcli_curl_ctx_destroy(struct gcli_ctx *ctx)
 {
 	if (ctx->curl)
 		curl_easy_cleanup(ctx->curl);
+
 	ctx->curl = NULL;
+
+	gcli_clear_ptr(&ctx->curl_useragent);
 }
 
 /* Ensures a clean cURL handle. Call this whenever you wanna use the
  * ctx->curl */
 static int
-gcli_curl_ensure(gcli_ctx *ctx)
+gcli_curl_ensure(struct gcli_ctx *ctx)
 {
 	if (ctx->curl) {
 		curl_easy_reset(ctx->curl);
@@ -80,14 +83,21 @@ gcli_curl_ensure(gcli_ctx *ctx)
 		    return gcli_error(ctx, "failed to initialise curl context");
 	}
 
+	if (!ctx->curl_useragent) {
+		curl_version_info_data const *ver;
+
+		ver = curl_version_info(CURLVERSION_NOW);
+		ctx->curl_useragent = gcli_asprintf("curl/%s", ver->version);
+	}
+
 	return 0;
 }
 
 /* Check the given curl code for an OK result. If not, print an
  * appropriate error message and exit */
 static int
-gcli_curl_check_api_error(gcli_ctx *ctx, CURLcode code, char const *url,
-                          gcli_fetch_buffer *const result)
+gcli_curl_check_api_error(struct gcli_ctx *ctx, CURLcode code, char const *url,
+                          struct gcli_fetch_buffer *const result)
 {
 	long status_code = 0;
 
@@ -108,7 +118,7 @@ gcli_curl_check_api_error(gcli_ctx *ctx, CURLcode code, char const *url,
 	return 0;
 }
 
-/* Callback for writing data into the gcli_fetch_buffer passed by
+/* Callback for writing data into the struct gcli_fetch_buffer passed by
  * calling routines */
 static size_t
 fetch_write_callback(char *in, size_t size, size_t nmemb, void *data)
@@ -116,7 +126,7 @@ fetch_write_callback(char *in, size_t size, size_t nmemb, void *data)
 	/* the user may have passed null indicating that we do not care
 	 * about the result body of the request. */
 	if (data) {
-		gcli_fetch_buffer *out = data;
+		struct gcli_fetch_buffer *out = data;
 
 		out->data = realloc(out->data, out->length + size * nmemb);
 		memcpy(&(out->data[out->length]), in, size * nmemb);
@@ -129,10 +139,10 @@ fetch_write_callback(char *in, size_t size, size_t nmemb, void *data)
 /* Plain HTTP get request.
  *
  * pagination_next returns the next url to query for paged results.
- * Results are placed into the gcli_fetch_buffer. */
+ * Results are placed into the struct gcli_fetch_buffer. */
 int
-gcli_fetch(gcli_ctx *ctx, char const *url, char **const pagination_next,
-           gcli_fetch_buffer *out)
+gcli_fetch(struct gcli_ctx *ctx, char const *url, char **const pagination_next,
+           struct gcli_fetch_buffer *out)
 {
 	return gcli_fetch_with_method(ctx, "GET", url, NULL, pagination_next, out);
 }
@@ -141,7 +151,7 @@ static int
 gcli_report_progress(void *_ctx, double dltotal, double dlnow,
                      double ultotal, double ulnow)
 {
-	gcli_ctx *ctx = _ctx;
+	struct gcli_ctx *ctx = _ctx;
 
 	(void) dltotal;
 	(void) dlnow;
@@ -156,10 +166,10 @@ gcli_report_progress(void *_ctx, double dltotal, double dlnow,
 
 /* Check the given url for a successful query */
 int
-gcli_curl_test_success(gcli_ctx *ctx, char const *url)
+gcli_curl_test_success(struct gcli_ctx *ctx, char const *url)
 {
 	CURLcode ret;
-	gcli_fetch_buffer buffer = {0};
+	struct gcli_fetch_buffer buffer = {0};
 	long status_code;
 	bool is_success = true;
 	int rc = 0;
@@ -171,7 +181,7 @@ gcli_curl_test_success(gcli_ctx *ctx, char const *url)
 	curl_easy_setopt(ctx->curl, CURLOPT_BUFFERSIZE, 102400L);
 	curl_easy_setopt(ctx->curl, CURLOPT_NOPROGRESS, 1L);
 	curl_easy_setopt(ctx->curl, CURLOPT_MAXREDIRS, 50L);
-	curl_easy_setopt(ctx->curl, CURLOPT_USERAGENT, "curl/7.78.0");
+	curl_easy_setopt(ctx->curl, CURLOPT_USERAGENT, ctx->curl_useragent);
 #if defined(CURL_HTTP_VERSION_2TLS)
 	curl_easy_setopt(
 		ctx->curl, CURLOPT_HTTP_VERSION, (long)CURL_HTTP_VERSION_2TLS);
@@ -203,7 +213,7 @@ gcli_curl_test_success(gcli_ctx *ctx, char const *url)
 	if (ctx->report_progress)
 		ctx->report_progress(true);
 
-	free(buffer.data);
+	gcli_fetch_buffer_free(&buffer);
 
 	return is_success;
 }
@@ -213,11 +223,12 @@ gcli_curl_test_success(gcli_ctx *ctx, char const *url)
  *
  * content_type may be NULL. */
 int
-gcli_curl(gcli_ctx *ctx, FILE *stream, char const *url, char const *content_type)
+gcli_curl(struct gcli_ctx *ctx, FILE *stream, char const *url,
+          char const *content_type)
 {
 	CURLcode ret;
 	struct curl_slist *headers;
-	gcli_fetch_buffer buffer = {0};
+	struct gcli_fetch_buffer buffer = {0};
 	char *auth_header = NULL;
 	int rc = 0;
 
@@ -230,7 +241,8 @@ gcli_curl(gcli_ctx *ctx, FILE *stream, char const *url, char const *content_type
 		headers = curl_slist_append(headers, content_type);
 
 	auth_header = gcli_get_authheader(ctx);
-	headers = curl_slist_append(headers, auth_header);
+	if (auth_header)
+		headers = curl_slist_append(headers, auth_header);
 
 	curl_easy_setopt(ctx->curl, CURLOPT_URL, url);
 	curl_easy_setopt(ctx->curl, CURLOPT_BUFFERSIZE, 102400L);
@@ -238,7 +250,7 @@ gcli_curl(gcli_ctx *ctx, FILE *stream, char const *url, char const *content_type
 	curl_easy_setopt(ctx->curl, CURLOPT_MAXREDIRS, 50L);
 	curl_easy_setopt(ctx->curl, CURLOPT_FTP_SKIP_PASV_IP, 1L);
 	curl_easy_setopt(ctx->curl, CURLOPT_HTTPHEADER, headers);
-	curl_easy_setopt(ctx->curl, CURLOPT_USERAGENT, "curl/7.78.0");
+	curl_easy_setopt(ctx->curl, CURLOPT_USERAGENT, ctx->curl_useragent);
 #if defined(CURL_HTTP_VERSION_2TLS)
 	curl_easy_setopt(
 		ctx->curl, CURLOPT_HTTP_VERSION, (long)CURL_HTTP_VERSION_2TLS);
@@ -265,11 +277,11 @@ gcli_curl(gcli_ctx *ctx, FILE *stream, char const *url, char const *content_type
 	if (rc == 0)
 		fwrite(buffer.data, 1, buffer.length, stream);
 
-	free(buffer.data);
+	gcli_fetch_buffer_free(&buffer);
 
 	curl_slist_free_all(headers);
 
-	free(auth_header);
+	gcli_clear_ptr(&auth_header);
 
 	return rc;
 }
@@ -280,17 +292,17 @@ fetch_header_callback(char *ptr, size_t size, size_t nmemb, void *userdata)
 {
 	char **out = userdata;
 
-	size_t sz          = size * nmemb;
-	sn_sv  buffer      = sn_sv_from_parts(ptr, sz);
-	sn_sv  header_name = sn_sv_chop_until(&buffer, ':');
+	size_t sz = size * nmemb;
+	gcli_sv buffer = gcli_sv_from_parts(ptr, sz);
+	gcli_sv header_name = gcli_sv_chop_until(&buffer, ':');
 
 	/* Despite what the documentation says, this header is called
 	 * "link" not "Link". Webdev ftw /sarc */
-	if (sn_sv_eq_to(header_name, "link")) {
-		buffer.data   += 1;
+	if (gcli_sv_eq_to(header_name, "link")) {
+		buffer.data += 1;
 		buffer.length -= 1;
-		buffer         = sn_sv_trim_front(buffer);
-		*out           = sn_strndup(buffer.data, buffer.length);
+		buffer = gcli_sv_trim_front(buffer);
+		*out = gcli_strndup(buffer.data, buffer.length);
 	}
 
 	return sz;
@@ -300,12 +312,12 @@ fetch_header_callback(char *ptr, size_t size, size_t nmemb, void *userdata)
 static char *
 parse_link_header(char *_header)
 {
-	sn_sv header = SV(_header);
-	sn_sv entry  = {0};
+	gcli_sv header = SV(_header);
+	gcli_sv entry  = {0};
 
 	/* Iterate through the comma-separated list of link relations */
-	while ((entry = sn_sv_chop_until(&header, ',')).length > 0) {
-		entry = sn_sv_trim(entry);
+	while ((entry = gcli_sv_chop_until(&header, ',')).length > 0) {
+		entry = gcli_sv_trim(entry);
 
 		/* the entries have semicolon-separated fields like so:
 		 * <url>; rel=\"next\"
@@ -315,20 +327,20 @@ parse_link_header(char *_header)
 		 * We're making lots of assumptions about the input data here
 		 * without sanity checking it. If it fails, we will know. Most
 		 * likely a segfault. */
-		sn_sv almost_url = sn_sv_chop_until(&entry, ';');
+		gcli_sv almost_url = gcli_sv_chop_until(&entry, ';');
 
-		if (sn_sv_eq_to(entry, "; rel=\"next\"")) {
+		if (gcli_sv_eq_to(entry, "; rel=\"next\"")) {
 			/* Skip the triangle brackets around the url */
-			almost_url.data   += 1;
+			almost_url.data += 1;
 			almost_url.length -= 2;
-			almost_url         = sn_sv_trim(almost_url);
-			return sn_sv_to_cstr(almost_url);
+			almost_url = gcli_sv_trim(almost_url);
+			return gcli_sv_to_cstr(almost_url);
 		}
 
 		/* skip the comma if we have enough data */
 		if (header.length > 0) {
 			header.length -= 1;
-			header.data   += 1;
+			header.data += 1;
 		}
 	}
 
@@ -350,17 +362,17 @@ parse_link_header(char *_header)
  * will be set to NULL. */
 int
 gcli_fetch_with_method(
-	gcli_ctx *ctx,
-	char const *method,         /* HTTP method. e.g. POST, GET, DELETE etc. */
-	char const *url,            /* Endpoint                                 */
-	char const *data,           /* Form data                                */
-	char **const pagination_next, /* Next URL for pagination                  */
-	gcli_fetch_buffer *const out) /* output buffer                            */
+	struct gcli_ctx *ctx,
+	char const *method,                  /* HTTP method. e.g. POST, GET, DELETE etc. */
+	char const *url,                     /* Endpoint */
+	char const *data,                    /* Form data */
+	char **const pagination_next,        /* Next URL for pagination */
+	struct gcli_fetch_buffer *const out) /* output buffer */
 {
 	CURLcode ret;
 	struct curl_slist *headers;
-	gcli_fetch_buffer tmp = {0}; /* used for error codes when out is NULL */
-	gcli_fetch_buffer *buf = NULL;
+	struct gcli_fetch_buffer tmp = {0}; /* used for error codes when out is NULL */
+	struct gcli_fetch_buffer *buf = NULL;
 	char *link_header = NULL;
 	int rc = 0;
 
@@ -369,7 +381,7 @@ gcli_fetch_with_method(
 
 	char *auth_header = gcli_get_authheader(ctx);
 
-	if (sn_verbose())
+	if (gcli_be_verbose(ctx))
 		fprintf(stderr, "info: cURL request %s %s...\n", method, url);
 
 	headers = NULL;
@@ -386,7 +398,7 @@ gcli_fetch_with_method(
 	 * user is not interested in the result we use a temporary buffer
 	 * for proper error reporting. */
 	if (out) {
-		*out = (gcli_fetch_buffer) {0};
+		*out = (struct gcli_fetch_buffer) {0};
 		buf = out;
 	} else {
 		buf = &tmp;
@@ -398,7 +410,7 @@ gcli_fetch_with_method(
 		curl_easy_setopt(ctx->curl, CURLOPT_POSTFIELDS, data);
 
 	curl_easy_setopt(ctx->curl, CURLOPT_HTTPHEADER, headers);
-	curl_easy_setopt(ctx->curl, CURLOPT_USERAGENT, "curl/7.79.1");
+	curl_easy_setopt(ctx->curl, CURLOPT_USERAGENT, ctx->curl_useragent);
 	curl_easy_setopt(ctx->curl, CURLOPT_CUSTOMREQUEST, method);
 	curl_easy_setopt(ctx->curl, CURLOPT_TCP_KEEPALIVE, 1L);
 	curl_easy_setopt(ctx->curl, CURLOPT_WRITEDATA, buf);
@@ -426,12 +438,10 @@ gcli_fetch_with_method(
 		if (link_header && pagination_next)
 			*pagination_next = parse_link_header(link_header);
 	} else if (out) { /* error happened and we have an output buffer */
-		free(out->data);
-		out->data = NULL;
-		out->length = 0;
+		gcli_fetch_buffer_free(out);
 	}
 
-	free(link_header);
+	gcli_clear_ptr(&link_header);
 
 	curl_slist_free_all(headers);
 	headers = NULL;
@@ -439,9 +449,9 @@ gcli_fetch_with_method(
 	/* if the user is not interested in the result, free the temporary
 	 * buffer */
 	if (!out)
-		free(tmp.data);
+		gcli_fetch_buffer_free(&tmp);
 
-	free(auth_header);
+	gcli_clear_ptr(&auth_header);
 
 	return rc;
 }
@@ -453,9 +463,9 @@ gcli_fetch_with_method(
  * content_type may not be NULL.
  */
 int
-gcli_post_upload(gcli_ctx *ctx, char const *url, char const *content_type,
+gcli_post_upload(struct gcli_ctx *ctx, char const *url, char const *content_type,
                  void *buffer, size_t const buffer_size,
-                 gcli_fetch_buffer *const out)
+                 struct gcli_fetch_buffer *const out)
 {
 	CURLcode ret;
 	struct curl_slist *headers;
@@ -466,19 +476,22 @@ gcli_post_upload(gcli_ctx *ctx, char const *url, char const *content_type,
 		return rc;
 
 	auth_header = gcli_get_authheader(ctx);
-	contenttype_header = sn_asprintf("Content-Type: %s",
-	                                 content_type);
-	contentsize_header = sn_asprintf("Content-Length: %zu",
-	                                 buffer_size);
+	contenttype_header = gcli_asprintf("Content-Type: %s",
+	                                   content_type);
+	contentsize_header = gcli_asprintf("Content-Length: %zu",
+	                                   buffer_size);
 
-	if (sn_verbose())
+	if (gcli_be_verbose(ctx))
 		fprintf(stderr, "info: cURL upload POST %s...\n", url);
 
 	headers = NULL;
 	headers = curl_slist_append(
 		headers,
 		"Accept: application/vnd.github.v3+json");
-	headers = curl_slist_append(headers, auth_header);
+
+	if (auth_header)
+		headers = curl_slist_append(headers, auth_header);
+
 	headers = curl_slist_append(headers, contenttype_header);
 	headers = curl_slist_append(headers, contentsize_header);
 
@@ -488,7 +501,7 @@ gcli_post_upload(gcli_ctx *ctx, char const *url, char const *content_type,
 	curl_easy_setopt(ctx->curl, CURLOPT_POSTFIELDSIZE, (long)buffer_size);
 
 	curl_easy_setopt(ctx->curl, CURLOPT_HTTPHEADER, headers);
-	curl_easy_setopt(ctx->curl, CURLOPT_USERAGENT, "curl/7.79.1");
+	curl_easy_setopt(ctx->curl, CURLOPT_USERAGENT, ctx->curl_useragent);
 	curl_easy_setopt(ctx->curl, CURLOPT_WRITEDATA, out);
 	curl_easy_setopt(ctx->curl, CURLOPT_WRITEFUNCTION, fetch_write_callback);
 
@@ -508,9 +521,9 @@ gcli_post_upload(gcli_ctx *ctx, char const *url, char const *content_type,
 	curl_slist_free_all(headers);
 	headers = NULL;
 
-	free(auth_header);
-	free(contentsize_header);
-	free(contenttype_header);
+	gcli_clear_ptr(&auth_header);
+	gcli_clear_ptr(&contentsize_header);
+	gcli_clear_ptr(&contenttype_header);
 
 	return rc;
 }
@@ -521,9 +534,9 @@ gcli_post_upload(gcli_ctx *ctx, char const *url, char const *content_type,
  *  code.
  */
 int
-gcli_curl_gitea_upload_attachment(gcli_ctx *ctx, char const *url,
+gcli_curl_gitea_upload_attachment(struct gcli_ctx *ctx, char const *url,
                                   char const *filename,
-                                  gcli_fetch_buffer *const out)
+                                  struct gcli_fetch_buffer *const out)
 {
 	CURLcode ret;
 	curl_mime *mime;
@@ -537,14 +550,16 @@ gcli_curl_gitea_upload_attachment(gcli_ctx *ctx, char const *url,
 
 	auth_header = gcli_get_authheader(ctx);
 
-	if (sn_verbose())
+	if (gcli_be_verbose(ctx))
 		fprintf(stderr, "info: cURL upload POST %s...\n", url);
 
 	headers = NULL;
 	headers = curl_slist_append(
 		headers,
 		"Accept: application/json");
-	headers = curl_slist_append(headers, auth_header);
+
+	if (auth_header)
+		headers = curl_slist_append(headers, auth_header);
 
 	/* The docs say we should be using this mime thing. */
 	mime = curl_mime_init(ctx->curl);
@@ -584,13 +599,13 @@ gcli_curl_gitea_upload_attachment(gcli_ctx *ctx, char const *url,
 	curl_slist_free_all(headers);
 	headers = NULL;
 	curl_mime_free(mime);
-	free(auth_header);
+	gcli_clear_ptr(&auth_header);
 
 	return rc;
 }
 
-sn_sv
-gcli_urlencode_sv(sn_sv const _input)
+gcli_sv
+gcli_urlencode_sv(gcli_sv const _input)
 {
 	size_t input_len;
 	size_t output_len;
@@ -613,18 +628,18 @@ gcli_urlencode_sv(sn_sv const _input)
 		}
 	}
 
-	return sn_sv_from_parts(output, output_len);
+	return gcli_sv_from_parts(output, output_len);
 }
 
 char *
 gcli_urlencode(char const *input)
 {
-	sn_sv encoded = gcli_urlencode_sv(SV((char *)input));
+	gcli_sv encoded = gcli_urlencode_sv(SV((char *)input));
 	return encoded.data;
 }
 
 char *
-gcli_urldecode(gcli_ctx *ctx, char const *input)
+gcli_urldecode(struct gcli_ctx *ctx, char const *input)
 {
 	char *curlresult, *result;
 
@@ -656,13 +671,13 @@ gcli_urldecode(gcli_ctx *ctx, char const *input)
  *
  * If max is -1 then everything will be fetched. */
 int
-gcli_fetch_list(gcli_ctx *ctx, char *url, gcli_fetch_list_ctx *fl)
+gcli_fetch_list(struct gcli_ctx *ctx, char *url, struct gcli_fetch_list_ctx *fl)
 {
 	char *next_url = NULL;
 	int rc;
 
 	do {
-		gcli_fetch_buffer buffer = {0};
+		struct gcli_fetch_buffer buffer = {0};
 
 		rc = gcli_fetch(ctx, url, &next_url, &buffer);
 		if (rc == 0) {
@@ -676,15 +691,25 @@ gcli_fetch_list(gcli_ctx *ctx, char *url, gcli_fetch_list_ctx *fl)
 			json_close(&stream);
 		}
 
-		free(buffer.data);
-		free(url);
+		gcli_fetch_buffer_free(&buffer);
+		gcli_clear_ptr(&url);
 
 		if (rc < 0)
 			break;
 
 	} while ((url = next_url) && (fl->max == -1 || (int)(*fl->sizep) < fl->max));
 
-	free(next_url);
+	gcli_clear_ptr(&next_url);
 
 	return rc;
 }
+
+void
+gcli_fetch_buffer_free(struct gcli_fetch_buffer *const buffer)
+{
+	if (!buffer)
+		return;
+
+	gcli_clear_ptr(&buffer->data);
+	buffer->length = 0;
+}
