@@ -1,5 +1,5 @@
 /*
- * Copyright 2022 Nico Sonack <nsonack@herrhotzenplotz.de>
+ * Copyright 2022-2025 Nico Sonack <nsonack@herrhotzenplotz.de>
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -34,6 +34,8 @@
 #include <gcli/curl.h>
 #include <gcli/github/checks.h>
 #include <gcli/github/checks.h>
+#include <gcli/github/path.h>
+#include <gcli/github/repos.h>
 #include <gcli/json_util.h>
 
 #include <templates/github/checks.h>
@@ -41,18 +43,25 @@
 #include <pdjson/pdjson.h>
 
 int
-github_get_checks(gcli_ctx *ctx, char const *owner, char const *repo,
-                  char const *ref, int const max, github_check_list *const out)
+github_get_checks(struct gcli_ctx *ctx, struct gcli_path const *const path,
+                  char const *ref, int const max, struct github_check_list *const out)
 {
-	gcli_fetch_buffer buffer = {0};
+	struct gcli_fetch_buffer buffer = {0};
+	struct gcli_path norm_path = {0};
 	char *url = NULL, *next_url = NULL;
 	int rc = 0;
 
 	assert(out);
 
-	url = sn_asprintf("%s/repos/%s/%s/commits/%s/check-runs",
-	                  gcli_get_apibase(ctx),
-	                  owner, repo, ref);
+	/* we must normalise these paths to default paths. otherwise we'd get
+	 * bugs when a URL path to some item is passed in. */
+	rc = github_path_normalise(ctx, path, &norm_path);
+	if (rc < 0)
+		return rc;
+
+	rc = github_repo_make_url(ctx, path, &url, "/commits/%s/check-runs", ref);
+	if (rc < 0)
+		return rc;
 
 	do {
 		rc = gcli_fetch(ctx, url, &next_url, &buffer);
@@ -64,31 +73,38 @@ github_get_checks(gcli_ctx *ctx, char const *owner, char const *repo,
 			json_close(&stream);
 		}
 
-		free(url);
-		free(buffer.data);
+		gcli_clear_ptr(&url);
+		gcli_fetch_buffer_free(&buffer);
 
 		if (rc < 0)
 			break;
 	} while ((url = next_url) && ((int)(out->checks_size) < max || max < 0));
 
 	/* TODO: don't leak list on error */
-	free(next_url);
+	gcli_clear_ptr(&next_url);
+
+	gcli_path_free(&norm_path);
 
 	return rc;
 }
 
 void
-github_free_checks(github_check_list *const list)
+gcli_github_check_free(struct gcli_github_check *check)
+{
+	gcli_clear_ptr(&check->name);
+	gcli_clear_ptr(&check->status);
+	gcli_clear_ptr(&check->conclusion);
+	gcli_clear_ptr(&check->started_at);
+	gcli_clear_ptr(&check->completed_at);
+}
+
+void
+github_free_checks(struct github_check_list *const list)
 {
 	for (size_t i = 0; i < list->checks_size; ++i) {
-		free(list->checks[i].name);
-		free(list->checks[i].status);
-		free(list->checks[i].conclusion);
-		free(list->checks[i].started_at);
-		free(list->checks[i].completed_at);
+		gcli_github_check_free(&list->checks[i]);
 	}
 
-	free(list->checks);
-	list->checks = NULL;
+	gcli_clear_ptr(&list->checks);
 	list->checks_size = 0;
 }
