@@ -1,5 +1,5 @@
 /*
- * Copyright 2021, 2022 Nico Sonack <nsonack@herrhotzenplotz.de>
+ * Copyright 2021-2025 Nico Sonack <nsonack@herrhotzenplotz.de>
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -34,71 +34,75 @@
 #include <gcli/cmd/table.h>
 
 #include <pdjson/pdjson.h>
-#include <sn/sn.h>
 
 #include <stdlib.h>
 
 #include <templates/gitlab/snippets.h>
 
 void
-gcli_snippets_free(gcli_gitlab_snippet_list *const list)
+gcli_gitlab_snippet_free(struct gcli_gitlab_snippet *snippet)
+{
+	gcli_clear_ptr(&snippet->title);
+	gcli_clear_ptr(&snippet->filename);
+	gcli_clear_ptr(&snippet->date);
+	gcli_clear_ptr(&snippet->author);
+	gcli_clear_ptr(&snippet->visibility);
+	gcli_clear_ptr(&snippet->raw_url);
+}
+
+void
+gcli_snippets_free(struct gcli_gitlab_snippet_list *const list)
 {
 	for (size_t i = 0; i < list->snippets_size; ++i) {
-		free(list->snippets[i].title);
-		free(list->snippets[i].filename);
-		free(list->snippets[i].date);
-		free(list->snippets[i].author);
-		free(list->snippets[i].visibility);
-		free(list->snippets[i].raw_url);
+		gcli_gitlab_snippet_free(&list->snippets[i]);
 	}
 
-	free(list->snippets);
-
-	list->snippets = NULL;
+	gcli_clear_ptr(&list->snippets);
 	list->snippets_size = 0;
 }
 
 int
-gcli_snippets_get(gcli_ctx *ctx, int const max, gcli_gitlab_snippet_list *const out)
+gcli_snippets_get(struct gcli_ctx *ctx, int const max,
+                  struct gcli_gitlab_snippet_list *const out)
 {
 	char *url = NULL;
 
-	gcli_fetch_list_ctx fl = {
+	struct gcli_fetch_list_ctx fl = {
 		.listp = &out->snippets,
 		.sizep = &out->snippets_size,
 		.max = max,
 		.parse = (parsefn)(parse_gitlab_snippets),
 	};
 
-	*out = (gcli_gitlab_snippet_list) {0};
-	url = sn_asprintf("%s/snippets", gcli_get_apibase(ctx));
+	*out = (struct gcli_gitlab_snippet_list) {0};
+	url = gcli_asprintf("%s/snippets", gcli_get_apibase(ctx));
 
 	return gcli_fetch_list(ctx, url, &fl);
 }
 
 int
-gcli_snippet_delete(gcli_ctx *ctx, char const *snippet_id)
+gcli_snippet_delete(struct  gcli_ctx *ctx, char const *snippet_id)
 {
 	int rc = 0;
 	char *url;
 
-	url = sn_asprintf("%s/snippets/%s", gcli_get_apibase(ctx), snippet_id);
+	url = gcli_asprintf("%s/snippets/%s", gcli_get_apibase(ctx), snippet_id);
 	rc = gcli_fetch_with_method(ctx, "DELETE", url, NULL, NULL, NULL);
 
-	free(url);
+	gcli_clear_ptr(&url);
 
 	return rc;
 }
 
 int
-gcli_snippet_get(gcli_ctx *ctx, char const *snippet_id, FILE *stream)
+gcli_snippet_get(struct gcli_ctx *ctx, char const *snippet_id, FILE *stream)
 {
 	int rc = 0;
-	char *url = sn_asprintf("%s/snippets/%s/raw",
-	                        gcli_get_apibase(ctx),
-	                        snippet_id);
+	char *url = gcli_asprintf("%s/snippets/%s/raw",
+	                          gcli_get_apibase(ctx),
+	                          snippet_id);
 	rc = gcli_curl(ctx, stream, url, NULL);
-	free(url);
+	gcli_clear_ptr(&url);
 
 	return rc;
 }
